@@ -4,10 +4,12 @@ use std::ffi::OsString;
 use std::fmt::Write;
 use std::path::PathBuf;
 
-use console::style;
+use cursive::theme::{BaseColor, Effect, Style};
+use cursive::utils::markup::StyledString;
 use eyre::Context;
 use tracing::{instrument, warn};
 
+use crate::core::formatting::StyledStringBuilder;
 use crate::git::{ConfigRead, GitRunInfo, GitRunOpts, Repo};
 
 use super::effects::Effects;
@@ -45,6 +47,16 @@ pub fn get_main_branch_name(repo: &Repo) -> eyre::Result<String> {
     Ok("master".to_string())
 }
 
+/// If `true`, switch to the branch associated with a target commit instead of
+/// the commit directly.
+///
+/// The switch will only occur if it is the only branch on the target commit.
+#[instrument]
+pub fn get_auto_switch_branches(repo: &Repo) -> eyre::Result<bool> {
+    repo.get_readonly_config()?
+        .get_or("branchless.navigation.autoSwitchBranches", true)
+}
+
 /// Get the default comment character.
 #[instrument]
 pub fn get_comment_char(repo: &Repo) -> eyre::Result<char> {
@@ -109,7 +121,7 @@ pub fn get_editor(git_run_info: &GitRunInfo, repo: &Repo) -> eyre::Result<Option
     if let Ok(result) =
         git_run_info.run_silent(repo, None, &["var", "GIT_EDITOR"], GitRunOpts::default())
     {
-        if result.exit_code == 0 {
+        if result.exit_code.is_success() {
             let editor =
                 std::str::from_utf8(&result.stdout).context("Decoding git var output as UTF-8")?;
             let editor = editor.trim_end();
@@ -121,7 +133,7 @@ pub fn get_editor(git_run_info: &GitRunInfo, repo: &Repo) -> eyre::Result<Option
     }
 
     let editor = std::env::var_os("GIT_EDITOR");
-    if editor != None {
+    if editor.is_some() {
         return Ok(editor);
     }
 
@@ -184,22 +196,30 @@ pub const RESTACK_WARN_ABANDONED_CONFIG_KEY: &str = "branchless.restack.warnAban
 /// Possible hint types.
 #[derive(Clone, Debug)]
 pub enum Hint {
+    /// Suggest running `git test clean` in order to clean cached test results.
+    CleanCachedTestResults,
+
+    /// Suggest omitting arguments when they would default to `HEAD`.
+    MoveImplicitHeadArgument,
+
     /// Suggest running `git restack` when a commit is abandoned as part of a `rewrite` event.
     RestackWarnAbandoned,
 
     /// Suggest running `git restack` when the smartlog prints an abandoned commit.
     SmartlogFixAbandoned,
 
-    /// Suggest omitting arguments when they would default to `HEAD`.
-    MoveImplicitHeadArgument,
+    /// Suggest showing more output with `git test show` using `--verbose`.
+    TestShowVerbose,
 }
 
 impl Hint {
     fn get_config_key(&self) -> &'static str {
         match self {
+            Hint::CleanCachedTestResults => "branchless.hint.cleanCachedTestResults",
+            Hint::MoveImplicitHeadArgument => "branchless.hint.moveImplicitHeadArgument",
             Hint::RestackWarnAbandoned => "branchless.hint.restackWarnAbandoned",
             Hint::SmartlogFixAbandoned => "branchless.hint.smartlogFixAbandoned",
-            Hint::MoveImplicitHeadArgument => "branchless.hint.moveImplicitHeadArgument",
+            Hint::TestShowVerbose => "branchless.hint.testShowVerbose",
         }
     }
 }
@@ -210,12 +230,22 @@ pub fn get_hint_enabled(repo: &Repo, hint: Hint) -> eyre::Result<bool> {
         .get_or(hint.get_config_key(), true)
 }
 
+/// Render the leading colored "hint" text for use in messaging.
+pub fn get_hint_string() -> StyledString {
+    StyledStringBuilder::new()
+        .append_styled(
+            "hint",
+            Style::merge(&[BaseColor::Blue.dark().into(), Effect::Bold.into()]),
+        )
+        .build()
+}
+
 /// Print instructions explaining how to disable a given hint.
 pub fn print_hint_suppression_notice(effects: &Effects, hint: Hint) -> eyre::Result<()> {
     writeln!(
         effects.get_output_stream(),
         "{}: disable this hint by running: git config --global {} false",
-        style("hint").blue().bold(),
+        effects.get_glyphs().render(get_hint_string())?,
         hint.get_config_key(),
     )?;
     Ok(())
