@@ -1,12 +1,13 @@
 //! Wrappers around various side effects.
 
+use bstr::ByteSlice;
 use std::convert::TryInto;
-use std::fmt::Write;
+use std::fmt::{Debug, Write};
 use std::io::{stderr, stdout, Stderr, Stdout, Write as WriteIo};
 use std::mem::take;
 use std::sync::{Arc, Mutex, RwLock};
-use std::thread;
 use std::time::{Duration, Instant};
+use std::{io, thread};
 
 use indicatif::{MultiProgress, ProgressBar, ProgressDrawTarget, ProgressStyle};
 use itertools::Itertools;
@@ -41,6 +42,8 @@ pub enum OperationType {
     RepairBranches,
     RepairCommits,
     RunGitCommand(Arc<String>),
+    RunTests(Arc<String>),
+    RunTestOnCommit(Arc<String>),
     SortCommits,
     SyncCommits,
     UpdateCommitGraph,
@@ -77,6 +80,8 @@ impl ToString for OperationType {
             OperationType::RunGitCommand(command) => {
                 return format!("Running Git command: {}", &command)
             }
+            OperationType::RunTests(command) => return format!("Running tests: {command}"),
+            OperationType::RunTestOnCommit(commit) => return format!("Waiting to test {commit}"),
             OperationType::SortCommits => "Sorting commits",
             OperationType::SyncCommits => "Syncing commit stacks",
             OperationType::UpdateCommitGraph => "Updating commit graph",
@@ -149,6 +154,8 @@ impl RootOperation {
                             operation_type: first.clone(),
                             progress_bar: ProgressBar::new_spinner(),
                             has_meter: Default::default(),
+                            icon: Default::default(),
+                            progress_message: first.to_string(),
                             start_times: Default::default(),
                             elapsed_duration: Default::default(),
                             children: Default::default(),
@@ -185,7 +192,7 @@ impl RootOperation {
     pub fn tick(&mut self) {
         let operations = {
             let mut acc = Vec::new();
-            self.traverse_operations(&mut acc, 0, &self.children);
+            Self::traverse_operations(&mut acc, 0, &self.children);
             acc
         };
         for (nesting_level, operation) in operations {
@@ -197,7 +204,7 @@ impl RootOperation {
     pub fn refresh_multi_progress(&mut self) {
         let operations = {
             let mut acc = Vec::new();
-            self.traverse_operations(&mut acc, 0, &self.children);
+            Self::traverse_operations(&mut acc, 0, &self.children);
             acc
         };
         if self.multi_progress.clear().is_err() {
@@ -220,24 +227,62 @@ impl RootOperation {
     }
 
     fn traverse_operations<'a>(
-        &self,
         acc: &mut Vec<(usize, &'a OperationState)>,
         current_level: usize,
         operations: &'a [OperationState],
     ) {
         for operation in operations {
             acc.push((current_level, operation));
-            self.traverse_operations(acc, current_level + 1, &operation.children);
+            Self::traverse_operations(acc, current_level + 1, &operation.children);
         }
     }
 }
 
+/// The string values associated with [`OperationIcon`]s.
+pub mod icons {
+    /// Used to indicate success.
+    pub const CHECKMARK: &str = "✓";
+
+    /// Used to indicate a warning.
+    pub const EXCLAMATION: &str = "⚠️";
+
+    /// Used to indicate failure.
+    ///
+    /// Can't use "✗️" in interactive progress meters because some terminals think its width is >1,
+    /// which seems to cause rendering issues because we use 1 as its width.
+    pub const CROSS: &str = "X";
+}
+
+/// An icon denoting the status of an operation.
+#[derive(Clone, Copy, Debug)]
+pub enum OperationIcon {
+    /// A suitable waiting icon should be rendered.
+    InProgress,
+
+    /// The operation was a success.
+    Success,
+
+    /// The operation produced a warning.
+    Warning,
+
+    /// The operation was a failure.
+    Failure,
+}
+
+impl Default for OperationIcon {
+    fn default() -> Self {
+        Self::InProgress
+    }
+}
+
 #[derive(Debug)]
 struct OperationState {
     operation_type: OperationType,
     progress_bar: ProgressBar,
     has_meter: bool,
     start_times: Vec<Instant>,
+    icon: OperationIcon,
+    progress_message: String,
     elapsed_duration: Duration,
     children: Vec<OperationState>,
 }
@@ -279,6 +324,8 @@ impl OperationState {
                             operation_type: first.clone(),
                             progress_bar: ProgressBar::new_spinner(),
                             has_meter: Default::default(),
+                            icon: Default::default(),
+                            progress_message: first.to_string(),
                             start_times: Default::default(),
                             elapsed_duration: Default::default(),
                             children: Default::default(),
@@ -312,7 +359,9 @@ impl OperationState {
 
     pub fn tick(&self, nesting_level: usize) {
         lazy_static! {
-            static ref CHECKMARK: String = console::style("✓").green().to_string();
+            static ref CHECKMARK: String = console::style(icons::CHECKMARK).green().to_string();
+            static ref EXCLAMATION: String = console::style(icons::EXCLAMATION).yellow().to_string();
+            static ref CROSS: String = console::style(icons::CROSS).red().to_string();
             static ref IN_PROGRESS_SPINNER_STYLE: Arc<Mutex<ProgressStyle>> =
                 Arc::new(Mutex::new(ProgressStyle::default_spinner().template("{prefix}{spinner} {wide_msg}").unwrap()));
             static ref IN_PROGRESS_BAR_STYLE: Arc<Mutex<ProgressStyle>> =
@@ -321,13 +370,32 @@ impl OperationState {
                 // the line so that the length number isn't overlapped by the
                 // cursor.
                 Arc::new(Mutex::new(ProgressStyle::default_bar().template("{prefix}{spinner} {wide_msg} {bar} {pos}/{len} ").unwrap()));
-            static ref FINISHED_PROGRESS_STYLE: Arc<Mutex<ProgressStyle>> = Arc::new(Mutex::new(IN_PROGRESS_SPINNER_STYLE
+            static ref WAITING_PROGRESS_STYLE: Arc<Mutex<ProgressStyle>> = Arc::new(Mutex::new(IN_PROGRESS_SPINNER_STYLE
+                .clone().lock().unwrap().clone()
+                // Requires at least two tick values, so just pass the same one twice.
+                .tick_strings(&[" ", " "])
+            ));
+            static ref SUCCESS_PROGRESS_STYLE: Arc<Mutex<ProgressStyle>> = Arc::new(Mutex::new(IN_PROGRESS_SPINNER_STYLE
                 .clone()
                 .lock()
                 .unwrap()
                 .clone()
                 // Requires at least two tick values, so just pass the same one twice.
                 .tick_strings(&[&CHECKMARK, &CHECKMARK])));
+            static ref WARNING_PROGRESS_STYLE: Arc<Mutex<ProgressStyle>> = Arc::new(Mutex::new(IN_PROGRESS_SPINNER_STYLE
+                .clone()
+                .lock()
+                .unwrap()
+                .clone()
+                // Requires at least two tick values, so just pass the same one twice.
+                .tick_strings(&[&CROSS, &CROSS])));
+            static ref FAILURE_PROGRESS_STYLE: Arc<Mutex<ProgressStyle>> = Arc::new(Mutex::new(IN_PROGRESS_SPINNER_STYLE
+                .clone()
+                .lock()
+                .unwrap()
+                .clone()
+                // Requires at least two tick values, so just pass the same one twice.
+                .tick_strings(&[&CROSS, &CROSS])));
         }
 
         let elapsed_duration = match self.start_times.iter().min() {
@@ -338,20 +406,29 @@ impl OperationState {
             }
         };
 
-        self.progress_bar
-            .set_style(match (self.start_times.as_slice(), self.has_meter) {
-                ([], _) => FINISHED_PROGRESS_STYLE.lock().unwrap().clone(),
-                ([..], false) => IN_PROGRESS_SPINNER_STYLE.lock().unwrap().clone(),
-                ([..], true) => IN_PROGRESS_BAR_STYLE.lock().unwrap().clone(),
-            });
-        // Both `set_message` and `set_prefix` implicitly call
-        // `ProgressBar::tick` and force a redraw.
+        self.progress_bar.set_style(
+            match (self.start_times.as_slice(), self.has_meter, self.icon) {
+                (_, _, OperationIcon::Success) => SUCCESS_PROGRESS_STYLE.lock().unwrap().clone(),
+                (_, _, OperationIcon::Warning) => WARNING_PROGRESS_STYLE.lock().unwrap().clone(),
+                (_, _, OperationIcon::Failure) => FAILURE_PROGRESS_STYLE.lock().unwrap().clone(),
+                ([], _, OperationIcon::InProgress) => {
+                    WAITING_PROGRESS_STYLE.lock().unwrap().clone()
+                }
+                ([..], false, OperationIcon::InProgress) => {
+                    IN_PROGRESS_SPINNER_STYLE.lock().unwrap().clone()
+                }
+                ([..], true, OperationIcon::InProgress) => {
+                    IN_PROGRESS_BAR_STYLE.lock().unwrap().clone()
+                }
+            },
+        );
         self.progress_bar.set_prefix("  ".repeat(nesting_level));
         self.progress_bar.set_message(format!(
             "{} ({:.1}s)",
-            self.operation_type.to_string(),
+            self.progress_message,
             elapsed_duration.as_secs_f64(),
         ));
+        self.progress_bar.tick();
     }
 }
 
@@ -557,7 +634,7 @@ impl Effects {
         operation_state.inc_progress(increment);
     }
 
-    fn on_set_message(&self, operation_key: &OperationKey, message: String) {
+    fn on_set_message(&self, operation_key: &OperationKey, icon: OperationIcon, message: String) {
         match self.dest {
             OutputDest::Stdout => {}
             OutputDest::Suppress | OutputDest::BufferForTest { .. } => return,
@@ -569,7 +646,8 @@ impl Effects {
             Some(operation_state) => operation_state,
             None => return,
         };
-        operation_state.progress_bar.set_message(message);
+        operation_state.icon = icon;
+        operation_state.progress_message = message;
     }
 
     fn on_drop_progress_handle(&self, operation_key: &OperationKey) {
@@ -585,7 +663,8 @@ impl Effects {
         let operation_state = match operation {
             Some(operation_state) => operation_state,
             None => {
-                warn!("Progress operation not started");
+                drop(root_operation); // Avoid potential deadlock.
+                warn!(?operation_key, "Progress operation not started");
                 return;
             }
         };
@@ -600,7 +679,11 @@ impl Effects {
         {
             Some(start_time_index) => operation_state.start_times.remove(start_time_index),
             None => {
-                warn!("Progress operation ended without matching start call");
+                drop(root_operation); // Avoid potential deadlock.
+                warn!(
+                    ?operation_key,
+                    "Progress operation ended without matching start call"
+                );
                 return;
             }
         };
@@ -841,7 +924,7 @@ impl Write for ErrorStream {
         match &self.dest {
             OutputDest::Stdout => {
                 self.buffer.push_str(s);
-                self.flush();
+                WriteProgress::flush(self);
             }
 
             OutputDest::Suppress => {
@@ -857,6 +940,33 @@ impl Write for ErrorStream {
     }
 }
 
+/// You probably don't want this. This implementation is only for `tracing`'s `fmt_layer`, because
+/// it needs a writer of type `io::Write`, but `Effects` normally uses its implementation of
+/// `fmt::Write`.
+impl io::Write for ErrorStream {
+    fn write(&mut self, buf: &[u8]) -> io::Result<usize> {
+        match &self.dest {
+            OutputDest::Stdout => {
+                self.buffer.push_str(buf.to_str_lossy().as_ref());
+                Ok(buf.len())
+            }
+            OutputDest::Suppress => {
+                // Do nothing.
+                Ok(buf.len())
+            }
+            OutputDest::BufferForTest { stdout: _, stderr } => {
+                let mut buffer = stderr.lock().unwrap();
+                buffer.write(buf)
+            }
+        }
+    }
+
+    fn flush(&mut self) -> io::Result<()> {
+        WriteProgress::flush(self);
+        Ok(())
+    }
+}
+
 impl Drop for ErrorStream {
     fn drop(&mut self) {
         WriteProgress::drop(self);
@@ -896,9 +1006,10 @@ impl ProgressHandle<'_> {
     }
 
     /// Update the message for this progress meter.
-    pub fn notify_status(&self, message: impl Into<String>) {
+    pub fn notify_status(&self, icon: OperationIcon, message: impl Into<String>) {
         let message = message.into();
-        self.effects.on_set_message(&self.operation_key, message);
+        self.effects
+            .on_set_message(&self.operation_key, icon, message);
     }
 }
 
