@@ -86,7 +86,10 @@ pub struct Glyphs {
     pub vertical_ellipsis: &'static str,
 
     /// Line used to connect a parent commit to its non-first child commit.
-    pub slash: &'static str,
+    pub split: &'static str,
+
+    /// Line used to connect a child commit to its non-first parent commit.
+    pub merge: &'static str,
 
     /// Cursor for a normal visible commit which is not currently checked out.
     pub commit_visible: &'static str,
@@ -116,6 +119,14 @@ pub struct Glyphs {
     /// currently checked out. (This is an unusual situation.)
     pub commit_main_obsolete_head: &'static str,
 
+    /// Cursor indicating that some number of commits have been omitted from the
+    /// smartlog at this position.
+    pub commit_omitted: &'static str,
+
+    /// Cursor indicating that a commit was either merging into this child
+    /// commit or merged from this parent commit.
+    pub commit_merge: &'static str,
+
     /// Character used to point to the currently-checked-out branch.
     pub branch_arrow: &'static str,
 
@@ -156,7 +167,8 @@ impl Glyphs {
             line: "|",
             line_with_offshoot: "|",
             vertical_ellipsis: ":",
-            slash: "\\",
+            split: "\\",
+            merge: "/",
             commit_visible: "o",
             commit_visible_head: "@",
             commit_obsolete: "x",
@@ -165,6 +177,8 @@ impl Glyphs {
             commit_main_head: "@",
             commit_main_obsolete: "X",
             commit_main_obsolete_head: "%",
+            commit_omitted: "#",
+            commit_merge: "&",
             branch_arrow: ">",
             bullet_point: "-",
             cycle_arrow: ">",
@@ -182,11 +196,14 @@ impl Glyphs {
             line: "┃",
             line_with_offshoot: "┣",
             vertical_ellipsis: "⋮",
-            slash: "━┓",
+            split: "━┓",
+            merge: "━┛",
             commit_visible: "◯",
             commit_visible_head: "●",
             commit_obsolete: "✕",
             commit_obsolete_head: "⦻",
+            commit_omitted: "⊘",
+            commit_merge: "↓",
             commit_main: "◇",
             commit_main_head: "◆",
             commit_main_obsolete: "✕",
@@ -200,6 +217,29 @@ impl Glyphs {
             cycle_lower_left_corner: "└",
         }
     }
+
+    /// Write the provided string to `out`, using ANSI escape codes as necessary to
+    /// style it.
+    ///
+    /// TODO: return something that implements `Display` instead of a `String`.
+    pub fn render(&self, string: StyledString) -> eyre::Result<String> {
+        let result = string
+            .spans()
+            .map(|span| {
+                let Span {
+                    content,
+                    attr,
+                    width: _,
+                } = span;
+                if self.should_write_ansi_escape_codes {
+                    Ok(render_style_as_ansi(content, *attr)?)
+                } else {
+                    Ok(content.to_string())
+                }
+            })
+            .collect::<eyre::Result<String>>()?;
+        Ok(result)
+    }
 }
 
 impl std::fmt::Debug for Glyphs {
@@ -379,26 +419,3 @@ fn render_style_as_ansi(content: &str, style: Style) -> eyre::Result<String> {
 
     Ok(output.to_string())
 }
-
-/// Write the provided string to `out`, using ANSI escape codes as necessary to
-/// style it.
-///
-/// TODO: return something that implements `Display` instead of a `String`.
-pub fn printable_styled_string(glyphs: &Glyphs, string: StyledString) -> eyre::Result<String> {
-    let result = string
-        .spans()
-        .map(|span| {
-            let Span {
-                content,
-                attr,
-                width: _,
-            } = span;
-            if glyphs.should_write_ansi_escape_codes {
-                Ok(render_style_as_ansi(content, *attr)?)
-            } else {
-                Ok(content.to_string())
-            }
-        })
-        .collect::<eyre::Result<String>>()?;
-    Ok(result)
-}
