@@ -82,10 +82,9 @@ pub fn find_abandoned_children(
         Some(MaybeZeroOid::Zero) => oid,
         None => return Ok(None),
     };
-
     let children = dag.query().children(CommitSet::from(oid))?;
-    let children = children.intersection(draft_commits);
-    let non_obsolete_children = children.difference(&dag.obsolete_commits);
+    let children = dag.filter_visible_commits(children)?;
+    let non_obsolete_children = children.difference(&dag.query_obsolete_commits());
     let non_obsolete_children_oids: Vec<NonZeroOid> = non_obsolete_children
         .iter()?
         .map(|x| -> eyre::Result<NonZeroOid> { NonZeroOid::try_from(x?) })
