@@ -11,7 +11,7 @@ use tracing::warn;
 use crate::core::check_out::{check_out_commit, CheckOutCommitOptions, CheckoutTarget};
 use crate::core::effects::Effects;
 use crate::core::eventlog::{EventLogDb, EventTransactionId};
-use crate::core::formatting::{printable_styled_string, Pluralize};
+use crate::core::formatting::Pluralize;
 use crate::core::repo_ext::RepoExt;
 use crate::git::{
     GitRunInfo, MaybeZeroOid, NonZeroOid, ReferenceName, Repo, ResolvedReferenceInfo,
@@ -30,6 +30,8 @@ pub fn move_branches<'a>(
     event_tx_id: EventTransactionId,
     rewritten_oids_map: &'a HashMap<NonZeroOid, MaybeZeroOid>,
 ) -> eyre::Result<()> {
+    let main_branch = repo.get_main_branch()?;
+    let main_branch_name = main_branch.get_reference_name()?;
     let branch_oid_to_names = repo.get_branch_oid_to_names()?;
 
     // We may experience an error in the case of a branch move. Ideally, we
@@ -78,22 +80,54 @@ pub fn move_branches<'a>(
 
             MaybeZeroOid::Zero => {
                 for name in names {
-                    match repo.find_reference(name) {
-                        Ok(Some(mut reference)) => {
-                            if let Err(err) = reference.delete() {
+                    if name == &main_branch_name {
+                        // Hack? Never delete the main branch. We probably got here by syncing the
+                        // main branch with the upstream version, but all main branch commits were
+                        // skipped. For a regular branch, we would delete the branch, but for the
+                        // main branch, we should update it to point directly to the upstream
+                        // version.
+                        let target_oid = match main_branch.get_upstream_branch_target()? {
+                            Some(target_oid) => {
+                                if let Err(err) = repo.create_reference(
+                                    &main_branch_name,
+                                    target_oid,
+                                    true,
+                                    "move main branch",
+                                ) {
+                                    branch_move_err = Some(eyre::eyre!(err));
+                                    break 'outer;
+                                }
+                                MaybeZeroOid::NonZero(target_oid)
+                            }
+                            None => {
+                                let mut main_branch_reference =
+                                    repo.get_main_branch()?.into_reference();
+                                if let Err(err) = main_branch_reference.delete() {
+                                    branch_move_err = Some(eyre::eyre!(err));
+                                    break 'outer;
+                                }
+                                MaybeZeroOid::Zero
+                            }
+                        };
+                        branch_moves.push((*old_oid, target_oid, name));
+                    } else {
+                        match repo.find_reference(name) {
+                            Ok(Some(mut reference)) => {
+                                if let Err(err) = reference.delete() {
+                                    branch_move_err = Some(eyre::eyre!(err));
+                                    break 'outer;
+                                }
+                            }
+                            Ok(None) => {
+                                warn!(?name, "Reference not found, not deleting")
+                            }
+                            Err(err) => {
                                 branch_move_err = Some(eyre::eyre!(err));
                                 break 'outer;
                             }
-                        }
-                        Ok(None) => {
-                            warn!(?name, "Reference not found, not deleting")
-                        }
-                        Err(err) => {
-                            branch_move_err = Some(eyre::eyre!(err));
-                            break 'outer;
-                        }
-                    };
-                    branch_moves.push((*old_oid, MaybeZeroOid::Zero, name));
+                        };
+                        branch_moves.push((*old_oid, MaybeZeroOid::Zero, name));
+                    }
                 }
             }
         }
@@ -313,8 +347,7 @@ impl MergeConflictInfo {
                 amount: self.conflicting_paths.len(),
                 unit: ("conflicting file", "conflicting files"),
             },
-            printable_styled_string(
-                effects.get_glyphs(),
+            effects.get_glyphs().render(
                 repo.friendly_describe_commit_from_oid(effects.get_glyphs(), self.commit_oid)?
             )?
         )?;
@@ -346,9 +379,8 @@ mod in_memory {
     use eyre::Context;
     use tracing::{instrument, warn};
 
-    use crate::core::effects::{Effects, OperationType};
+    use crate::core::effects::{Effects, OperationIcon, OperationType};
     use crate::core::eventlog::EventLogDb;
-    use crate::core::formatting::printable_styled_string;
     use crate::core::gc::mark_commit_reachable;
     use crate::core::rewrite::execute::check_out_updated_head;
     use crate::core::rewrite::move_branches;
@@ -404,6 +436,7 @@ mod in_memory {
                     | RebaseCommand::CreateLabel { .. }
                     | RebaseCommand::Reset { .. }
                     | RebaseCommand::Pick { .. }
+                    | RebaseCommand::Break
                     | RebaseCommand::RegisterExtraPostRewriteHook
                     | RebaseCommand::DetectEmptyCommit { .. }
                     | RebaseCommand::SkipUpstreamAppliedCommit { .. } => None,
@@ -450,6 +483,7 @@ mod in_memory {
             .filter(|command| match command {
                 RebaseCommand::CreateLabel { .. }
                 | RebaseCommand::Reset { .. }
+                | RebaseCommand::Break
                 | RebaseCommand::RegisterExtraPostRewriteHook
                 | RebaseCommand::DetectEmptyCommit { .. } => false,
                 RebaseCommand::Pick { .. }
@@ -492,10 +526,9 @@ mod in_memory {
                         .wrap_err("Finding commit to apply")?;
                     i += 1;
 
-                    let commit_description = printable_styled_string(
-                        effects.get_glyphs(),
-                        commit_to_apply.friendly_describe(effects.get_glyphs())?,
-                    )?;
+                    let commit_description = effects
+                        .get_glyphs()
+                        .render(commit_to_apply.friendly_describe(effects.get_glyphs())?)?;
                     let commit_num = format!("[{}/{}]", i, num_picks);
                     progress.notify_progress(i, num_picks);
 
@@ -509,10 +542,10 @@ mod in_memory {
                         });
                     };
 
-                    progress.notify_status(format!(
-                        "Applying patch for commit: {}",
-                        commit_description
-                    ));
+                    progress.notify_status(
+                        OperationIcon::InProgress,
+                        format!("Applying patch for commit: {}", commit_description),
+                    );
                     let commit_tree = match repo.cherry_pick_fast(
                         &commit_to_apply,
                         &current_commit,
@@ -538,8 +571,10 @@ mod in_memory {
                         )
                     })?;
 
-                    progress
-                        .notify_status(format!("Committing to repository: {}", commit_description));
+                    progress.notify_status(
+                        OperationIcon::InProgress,
+                        format!("Committing to repository: {}", commit_description),
+                    );
                     let committer_signature = if *preserve_timestamps {
                         commit_to_apply.get_committer()
                     } else {
@@ -559,13 +594,13 @@ mod in_memory {
                     let rebased_commit = repo
                         .find_commit_or_fail(rebased_commit_oid)
                         .wrap_err("Looking up just-rebased commit")?;
-                    let commit_description = printable_styled_string(
-                        effects.get_glyphs(),
-                        repo.friendly_describe_commit_from_oid(
-                            effects.get_glyphs(),
-                            rebased_commit_oid,
-                        )?,
-                    )?;
+                    let commit_description =
+                        effects
+                            .get_glyphs()
+                            .render(repo.friendly_describe_commit_from_oid(
+                                effects.get_glyphs(),
+                                rebased_commit_oid,
+                            )?)?;
                     if rebased_commit.is_empty() {
                         rewritten_oids.push((*original_commit_oid, MaybeZeroOid::Zero));
                         maybe_set_skipped_head_new_oid(*original_commit_oid, current_oid);
@@ -620,15 +655,16 @@ mod in_memory {
                         .wrap_err("Finding commit to apply")?;
                     i += 1;
 
-                    let commit_description = printable_styled_string(
-                        effects.get_glyphs(),
-                        commit_to_apply.friendly_describe(effects.get_glyphs())?,
-                    )?;
+                    let commit_description = effects
+                        .get_glyphs()
+                        .render(commit_to_apply.friendly_describe(effects.get_glyphs())?)?;
                     let commit_num = format!("[{}/{}]", i, num_picks);
                     progress.notify_progress(i, num_picks);
 
-                    progress
-                        .notify_status(format!("Applying merge commit: {}", commit_description));
+                    progress.notify_status(
+                        OperationIcon::InProgress,
+                        format!("Applying merge commit: {}", commit_description),
+                    );
                     let commit = repo.find_commit_or_fail(*replacement_commit_oid)?;
                     let commit_tree = commit.get_tree()?;
                     let commit_message = commit_to_apply.get_message_raw()?;
@@ -639,8 +675,10 @@ mod in_memory {
                         )
                     })?;
 
-                    progress
-                        .notify_status(format!("Committing to repository: {}", commit_description));
+                    progress.notify_status(
+                        OperationIcon::InProgress,
+                        format!("Committing to repository: {}", commit_description),
+                    );
                     let committer_signature = if *preserve_timestamps {
                         commit_to_apply.get_committer()
                     } else {
@@ -675,13 +713,13 @@ mod in_memory {
                         )
                         .wrap_err("Applying rebased commit")?;
 
-                    let commit_description = printable_styled_string(
-                        effects.get_glyphs(),
-                        repo.friendly_describe_commit_from_oid(
-                            effects.get_glyphs(),
-                            rebased_commit_oid,
-                        )?,
-                    )?;
+                    let commit_description =
+                        effects
+                            .get_glyphs()
+                            .render(repo.friendly_describe_commit_from_oid(
+                                effects.get_glyphs(),
+                                rebased_commit_oid,
+                            )?)?;
                     rewritten_oids.push((*commit_oid, MaybeZeroOid::NonZero(rebased_commit_oid)));
                     current_oid = rebased_commit_oid;
 
@@ -693,6 +731,10 @@ mod in_memory {
                     )?;
                 }
 
+                RebaseCommand::Break => {
+                    eyre::bail!("`break` not supported for in-memory rebases");
+                }
+
                 RebaseCommand::SkipUpstreamAppliedCommit { commit_oid } => {
                     i += 1;
                     let commit_num = format!("[{}/{}]", i, num_picks);
@@ -702,8 +744,7 @@ mod in_memory {
                     maybe_set_skipped_head_new_oid(*commit_oid, current_oid);
 
                     let commit_description = commit.friendly_describe(effects.get_glyphs())?;
-                    let commit_description =
-                        printable_styled_string(effects.get_glyphs(), commit_description)?;
+                    let commit_description = effects.get_glyphs().render(commit_description)?;
                     writeln!(
                         effects.get_output_stream(),
                         "{} Skipped commit (was already applied upstream): {}",
@@ -1184,8 +1225,7 @@ pub fn execute_rebase_plan(
                 writeln!(
                     effects.get_output_stream(),
                     "The merge commit was: {}",
-                    printable_styled_string(
-                        effects.get_glyphs(),
+                    effects.get_glyphs().render(
                         repo.friendly_describe_commit_from_oid(effects.get_glyphs(), commit_oid)?
                     )?,
                 )?;
@@ -1212,8 +1252,7 @@ pub fn execute_rebase_plan(
                 writeln!(
                     effects.get_output_stream(),
                     "The conflicting commit was: {}",
-                    printable_styled_string(
-                        effects.get_glyphs(),
+                    effects.get_glyphs().render(
                         repo.friendly_describe_commit_from_oid(effects.get_glyphs(), commit_oid)?
                     )?,
                 )?;
