@@ -13,14 +13,18 @@ use tracing::{instrument, warn};
 
 use crate::core::dag::{commit_set_to_vec, union_all, CommitSet, Dag};
 use crate::core::effects::{Effects, OperationType};
-use crate::core::formatting::{printable_styled_string, Pluralize};
+use crate::core::formatting::Pluralize;
 use crate::core::rewrite::{RepoPool, RepoResource};
 use crate::core::task::ResourcePool;
 use crate::git::{Commit, NonZeroOid, PatchId, Repo};
 
+/// Represents the target for certain [`RebaseCommand`]s.
 #[derive(Debug)]
 pub enum OidOrLabel {
+    /// A commit hash to check out directly.
     Oid(NonZeroOid),
+
+    /// A label created previously with [`RebaseCommand::CreateLabel`].
     Label(String),
 }
 
@@ -38,18 +42,31 @@ impl Display for OidOrLabel {
 pub enum RebaseCommand {
     /// Create a label (a reference stored in `refs/rewritten/`) pointing to the
     /// current rebase head for later use.
-    CreateLabel { label_name: String },
+    CreateLabel {
+        /// The new label name.
+        label_name: String,
+    },
 
     /// Move the rebase head to the provided label or commit.
-    Reset { target: OidOrLabel },
+    Reset {
+        /// The target to check out.
+        target: OidOrLabel,
+    },
 
     /// Apply the provided commit on top of the rebase head, and update the
     /// rebase head to point to the newly-applied commit.
     Pick {
+        /// The original commit, which contains the relevant metadata such as
+        /// the commit message.
         original_commit_oid: NonZeroOid,
+
+        /// The commit whose patch should be applied to the rebase head. This
+        /// will be different from [`original_commit_oid`] when a commit is
+        /// being replaced.
         commit_to_apply_oid: NonZeroOid,
     },
 
+    /// Merge two or more parent commits.
     Merge {
         /// If specified, the new merge commit will use this commit's message
         /// and tree, rather than attempting a new merge.
@@ -65,25 +82,40 @@ pub enum RebaseCommand {
         commits_to_merge: Vec<OidOrLabel>,
     },
 
+    /// Pause the rebase, to be resumed later. Only supported in on-disk
+    /// rebases.
+    Break,
+
     /// On-disk rebases only. Register that we want to run cleanup at the end of
     /// the rebase, during the `post-rewrite` hook.
     RegisterExtraPostRewriteHook,
 
     /// Determine if the current commit is empty. If so, reset the rebase head
     /// to its parent and record that it was empty in the `rewritten-list`.
-    DetectEmptyCommit { commit_oid: NonZeroOid },
+    DetectEmptyCommit {
+        /// The original commit. If the new commit is empty, then the original
+        /// commit will be recorded as skipped.
+        commit_oid: NonZeroOid,
+    },
 
     /// The commit that would have been applied to the rebase head was already
     /// applied upstream. Skip it and record it in the `rewritten-list`.
-    SkipUpstreamAppliedCommit { commit_oid: NonZeroOid },
+    SkipUpstreamAppliedCommit {
+        /// The original commit, which will be recorded as skipped.
+        commit_oid: NonZeroOid,
+    },
 }
 
 /// Represents a sequence of commands that can be executed to carry out a rebase
 /// operation.
 #[derive(Debug)]
 pub struct RebasePlan {
-    pub(super) first_dest_oid: NonZeroOid,
-    pub(super) commands: Vec<RebaseCommand>,
+    /// The first commit OID that will be checked out. This is necessary to
+    /// support on-disk rebases.
+    pub first_dest_oid: NonZeroOid,
+
+    /// The commands to run.
+    pub commands: Vec<RebaseCommand>,
 }
 
 impl ToString for RebaseCommand {
@@ -122,6 +154,7 @@ impl ToString for RebaseCommand {
                 // triggered.
                 format!("exec git show -s --format=%B {replacement_commit_oid} | git commit-tree -p {commit_oid} {other_parents} {replacement_commit_oid}^{{tree}}")
             }
+            RebaseCommand::Break => "break".to_string(),
             RebaseCommand::RegisterExtraPostRewriteHook => {
                 "exec git branchless hook-register-extra-post-rewrite-hook".to_string()
             }
@@ -159,7 +192,7 @@ impl<'a> RebasePlanPermissions<'a> {
         // error message which indicates the magnitude of the issue.
         let commits = dag.query().descendants(commits.clone())?;
 
-        let public_commits = dag.query_public_commits()?;
+        let public_commits = dag.query_public_commits_slow()?;
         if !build_options.force_rewrite_public_commits {
             let public_commits_to_move = public_commits.intersection(&commits);
             if !public_commits_to_move.is_empty()? {
@@ -169,13 +202,9 @@ impl<'a> RebasePlanPermissions<'a> {
             }
         }
 
-        let allowed_commits = dag.query().range(
-            commits.clone(),
-            commits.union(&dag.query_active_heads(&public_commits, &dag.observed_commits)?),
-        )?;
         Ok(Ok(RebasePlanPermissions {
             build_options,
-            allowed_commits,
+            allowed_commits: commits,
         }))
     }
 
@@ -274,8 +303,8 @@ impl<'a> ConstraintGraph<'a> {
                         .dag
                         .query()
                         .children(CommitSet::from(*children_of_oid))?
-                        .difference(&self.dag.obsolete_commits)
                         .difference(&commits_to_move);
+                    let source_children = self.dag.filter_visible_commits(source_children)?;
 
                     for child_oid in commit_set_to_vec(&source_children)? {
                         self.inner.entry(parent_oid).or_default().insert(child_oid);
@@ -329,21 +358,13 @@ impl<'a> ConstraintGraph<'a> {
         let _effects = effects;
 
         let all_descendants_of_constrained_nodes = {
-            let public_commits = self.dag.query_public_commits()?;
-            let active_heads = self.dag.query_active_heads(
-                &public_commits,
-                &self
-                    .dag
-                    .observed_commits
-                    .difference(&self.dag.obsolete_commits),
-            )?;
-            let visible_commits = self.dag.query().ancestors(active_heads)?;
+            let visible_commits = self.dag.query_visible_commits_slow()?;
 
             let mut acc = Vec::new();
             let parents = self.commits_to_move();
             progress.notify_progress(0, parents.len());
             for parent_oid in parents {
-                self.collect_descendants(&visible_commits, &mut acc, parent_oid)?;
+                self.collect_descendants(visible_commits, &mut acc, parent_oid)?;
                 progress.notify_progress_inc(1);
             }
             acc
@@ -525,7 +546,7 @@ enum Constraint {
 }
 
 /// Options used to build a rebase plan.
-#[derive(Debug)]
+#[derive(Clone, Debug)]
 pub struct BuildRebasePlanOptions {
     /// Force rewriting public commits, even though other users may have access
     /// to those commits.
@@ -600,10 +621,7 @@ impl BuildRebasePlanError {
                         char1,
                         char2,
                         char3,
-                        printable_styled_string(
-                            glyphs,
-                            repo.friendly_describe_commit_from_oid(glyphs, *oid)?,
-                        )?,
+                        glyphs.render(repo.friendly_describe_commit_from_oid(glyphs, *oid)?,)?,
                     )?;
                 }
             }
@@ -628,10 +646,9 @@ Retry with -f/--force-rewrite to proceed anyways.",
                         amount: public_commits_to_move.count()?,
                         unit: ("public commit", "public commits")
                     },
-                    printable_styled_string(
-                        effects.get_glyphs(),
-                        example_bad_commit.friendly_describe(effects.get_glyphs())?
-                    )?,
+                    effects
+                        .get_glyphs()
+                        .render(example_bad_commit.friendly_describe(effects.get_glyphs())?)?,
                 )?;
             }
 
@@ -1061,21 +1078,22 @@ impl<'a> RebasePlanBuilder<'a> {
     ) {
         let included_commit_oids: HashSet<NonZeroOid> = rebase_commands
             .iter()
-            .filter_map(|rebase_command| match rebase_command {
+            .flat_map(|rebase_command| match rebase_command {
                 RebaseCommand::CreateLabel { label_name: _ }
                 | RebaseCommand::Reset { target: _ }
+                | RebaseCommand::Break
                 | RebaseCommand::RegisterExtraPostRewriteHook
-                | RebaseCommand::DetectEmptyCommit { commit_oid: _ } => None,
+                | RebaseCommand::DetectEmptyCommit { commit_oid: _ } => Vec::new(),
                 RebaseCommand::Pick {
-                    original_commit_oid: _,
-                    commit_to_apply_oid: commit_oid,
-                }
-                | RebaseCommand::Merge {
+                    original_commit_oid,
+                    commit_to_apply_oid,
+                } => vec![*original_commit_oid, *commit_to_apply_oid],
+                RebaseCommand::Merge {
                     replacement_commit_oid: _,
                     commit_oid,
                     commits_to_merge: _,
                 }
-                | RebaseCommand::SkipUpstreamAppliedCommit { commit_oid } => Some(*commit_oid),
+                | RebaseCommand::SkipUpstreamAppliedCommit { commit_oid } => vec![*commit_oid],
             })
             .collect();
         let missing_commit_oids = state
@@ -1210,7 +1228,7 @@ impl<'a> RebasePlanBuilder<'a> {
                         .map(|commit_oid| match touched_paths_cache.get(&commit_oid) {
                             Some(upstream_touched_paths) => {
                                 if Self::should_check_patch_id(
-                                    &*upstream_touched_paths,
+                                    &upstream_touched_paths,
                                     &local_touched_paths,
                                 ) {
                                     CacheLookupResult::Cached(Some(commit_oid))
@@ -1937,11 +1955,12 @@ mod tests {
         x 70deb1e (rewritten as b8f27a86) create test3.txt
         |\
         | x 355e173 (rewritten as 22cf4586) create test4.txt
-        | |
-        | o 8fb706a Merge commit '355e173bf9c5d2efac2e451da0cdad3fb82b869a' into HEAD
+        | & (merge) 8fb706a Merge commit '355e173bf9c5d2efac2e451da0cdad3fb82b869a' into HEAD
         |
         x 9ea1b36 (rewritten as 2b47b505) create test5.txt
         |
+        | & (merge) 355e173 (rewritten as 22cf4586) create test4.txt
+        |/
         o 8fb706a Merge commit '355e173bf9c5d2efac2e451da0cdad3fb82b869a' into HEAD
         hint: there is 1 abandoned commit in your commit graph
         hint: to fix this, run: git restack
