@@ -9,7 +9,6 @@ use std::path::Path;
 use std::time::SystemTime;
 
 use console::style;
-use eden_dag::DagAlgorithm;
 use eyre::Context;
 use itertools::Itertools;
 use tempfile::NamedTempFile;
@@ -20,7 +19,7 @@ use crate::core::config::{get_hint_enabled, print_hint_suppression_notice, Hint}
 use crate::core::dag::Dag;
 use crate::core::effects::Effects;
 use crate::core::eventlog::{Event, EventLogDb, EventReplayer};
-use crate::core::formatting::{printable_styled_string, Pluralize};
+use crate::core::formatting::Pluralize;
 use crate::core::repo_ext::RepoExt;
 use crate::git::{
     CategorizedReferenceName, GitRunInfo, MaybeZeroOid, NonZeroOid, ReferenceName, Repo,
@@ -214,12 +213,7 @@ fn warn_abandoned(
         &references_snapshot,
     )?;
 
-    let public_commits = dag.query_public_commits()?;
-    let active_heads = dag.query_active_heads(
-        &public_commits,
-        &dag.observed_commits.difference(&dag.obsolete_commits),
-    )?;
-    let draft_commits = dag.query().range(public_commits, active_heads)?;
+    let draft_commits = dag.query_draft_commits()?;
 
     let (all_abandoned_children, all_abandoned_branches) = {
         let mut all_abandoned_children: HashSet<NonZeroOid> = HashSet::new();
@@ -227,7 +221,7 @@ fn warn_abandoned(
         for old_commit_oid in old_commit_oids {
             let abandoned_result = find_abandoned_children(
                 &dag,
-                &draft_commits,
+                draft_commits,
                 &event_replayer,
                 event_cursor,
                 old_commit_oid,
@@ -459,10 +453,9 @@ pub fn hook_drop_commit_if_empty(
     writeln!(
         effects.get_output_stream(),
         "Skipped now-empty commit: {}",
-        printable_styled_string(
-            effects.get_glyphs(),
-            head_commit.friendly_describe(effects.get_glyphs())?
-        )?
+        effects
+            .get_glyphs()
+            .render(head_commit.friendly_describe(effects.get_glyphs())?)?
     )?;
     repo.set_head(only_parent_oid)?;
 
@@ -498,10 +491,9 @@ pub fn hook_skip_upstream_applied_commit(
     writeln!(
         effects.get_output_stream(),
         "Skipping commit (was already applied upstream): {}",
-        printable_styled_string(
-            effects.get_glyphs(),
-            commit.friendly_describe(effects.get_glyphs())?
-        )?
+        effects
+            .get_glyphs()
+            .render(commit.friendly_describe(effects.get_glyphs())?)?
     )?;
 
     if let Some(orig_head_reference) = repo.find_reference(&"ORIG_HEAD".into())? {
