@@ -444,7 +444,7 @@ pub fn make_empty_tree(repo: &Repo) -> Result<Tree> {
 /// which wouldn't exist in one of the pre-/post-patch trees.
 fn remove_entry_if_exists(builder: &mut git2::TreeBuilder, name: &Path) -> Result<()> {
     if builder
-        .get(&name)
+        .get(name)
         .map_err(|err| Error::ReadTreeBuilderEntry {
             source: err,
             path: name.to_owned(),
@@ -452,7 +452,7 @@ fn remove_entry_if_exists(builder: &mut git2::TreeBuilder, name: &Path) -> Resul
         .is_some()
     {
         builder
-            .remove(&name)
+            .remove(name)
             .map_err(|err| Error::DeleteTreeBuilderEntry {
                 source: err,
                 path: name.to_owned(),
@@ -511,10 +511,10 @@ mod tests {
 
         git.init_repo()?;
 
-        git.write_file("foo", "foo")?;
-        git.write_file("bar/bar", "bar")?;
-        git.write_file("bar/baz", "qux")?;
-        git.write_file("xyzzy", "xyzzy")?;
+        git.write_file_txt("foo", "foo")?;
+        git.write_file_txt("bar/bar", "bar")?;
+        git.write_file_txt("bar/baz", "qux")?;
+        git.write_file_txt("xyzzy", "xyzzy")?;
         git.run(&["add", "."])?;
         git.run(&["commit", "-m", "commit"])?;
 
