@@ -104,7 +104,7 @@ impl Git {
         let repo_path = repo_path
             .to_str()
             .ok_or_else(|| eyre::eyre!("Could not convert repo path to string"))?;
-        let output = output.replace(&repo_path, "<repo-path>");
+        let output = output.replace(repo_path, "<repo-path>");
 
         lazy_static! {
             // Simulate clearing the terminal line by searching for the
@@ -350,14 +350,21 @@ stderr:
         Ok(())
     }
 
+    /// Write the provided contents to the provided file in the repository root.
+    /// For historical reasons, the name is suffixed with `.txt` (this is
+    /// technical debt).
+    pub fn write_file_txt(&self, name: &str, contents: &str) -> eyre::Result<()> {
+        let name = format!("{name}.txt");
+        self.write_file(&name, contents)
+    }
+
     /// Write the provided contents to the provided file in the repository root.
     pub fn write_file(&self, name: &str, contents: &str) -> eyre::Result<()> {
-        let path = PathBuf::from(name);
+        let path = self.repo_path.join(name);
         if let Some(dir) = path.parent() {
             std::fs::create_dir_all(self.repo_path.join(dir))?;
         }
-        let file_path = self.repo_path.join(format!("{}.txt", name));
-        std::fs::write(&file_path, contents)?;
+        std::fs::write(&path, contents)?;
         Ok(())
     }
 
@@ -388,7 +395,7 @@ stderr:
         time: isize,
         contents: &str,
     ) -> eyre::Result<NonZeroOid> {
-        self.write_file(name, contents)?;
+        self.write_file_txt(name, contents)?;
         self.run(&["add", "."])?;
         self.run_with_options(
             &["commit", "-m", &format!("create {}.txt", name)],
@@ -604,3 +611,25 @@ pub fn make_git_with_remote_repo() -> eyre::Result<GitWrapperWithRemoteRepo> {
         cloned_repo,
     })
 }
+
+/// Represents a Git worktree for an existing Git repository on disk.
+pub struct GitWorktreeWrapper {
+    /// Guard to clean up the containing temporary directory. Make sure to bind
+    /// this to a local variable not named `_`.
+    pub temp_dir: TempDir,
+
+    /// A wrapper around the worktree.
+    pub worktree: Git,
+}
+
+/// Create a new worktree for the provided repository.
+pub fn make_git_worktree(git: &Git, worktree_name: &str) -> eyre::Result<GitWorktreeWrapper> {
+    let temp_dir = tempfile::tempdir()?;
+    let worktree_path = temp_dir.path().join(worktree_name);
+    git.run(&["worktree", "add", worktree_path.to_str().unwrap()])?;
+    let worktree = Git {
+        repo_path: worktree_path,
+        ..git.clone()
+    };
+    Ok(GitWorktreeWrapper { temp_dir, worktree })
+}
