@@ -61,19 +61,20 @@ echo 'branchless: This is a bug. Please report it.'
 
 const ALL_ALIASES: &[(&str, &str)] = &[
     ("amend", "amend"),
-    ("co", "checkout"),
     ("hide", "hide"),
     ("move", "move"),
     ("next", "next"),
     ("prev", "prev"),
     ("query", "query"),
-    ("restack", "restack"),
     ("record", "record"),
+    ("restack", "restack"),
     ("reword", "reword"),
     ("sl", "smartlog"),
     ("smartlog", "smartlog"),
     ("submit", "submit"),
+    ("sw", "switch"),
     ("sync", "sync"),
+    ("test", "test"),
     ("undo", "undo"),
     ("unhide", "unhide"),
 ];
@@ -432,7 +433,7 @@ fn set_configs(
                 )?;
                 writeln!(
                     effects.get_output_stream(),
-                    "If this is incorrect, run: git config branchless.core.mainBranch <branch>"
+                    "If this is incorrect, run: git branchless init --main-branch <branch>"
                 )?;
                 main_branch_name
             }
@@ -556,7 +557,9 @@ pub fn init(
     main_branch_name: Option<&str>,
 ) -> eyre::Result<()> {
     let mut in_ = BufReader::new(stdin());
-    let mut repo = Repo::from_current_dir()?;
+    let repo = Repo::from_current_dir()?;
+    let mut repo = repo.open_worktree_parent_repo()?.unwrap_or(repo);
+
     let default_config = Config::open_default()?;
     let readonly_config = repo.get_readonly_config()?;
     let mut config = create_isolated_config(effects, &repo, readonly_config.into_config())?;
