@@ -17,6 +17,7 @@ mod smartlog;
 mod snapshot;
 mod submit;
 mod sync;
+mod test;
 mod undo;
 mod wrap;
 
@@ -32,22 +33,24 @@ use cursive::theme::BaseColor;
 use cursive::utils::markup::StyledString;
 use eyre::Context;
 use itertools::Itertools;
-use lib::core::formatting::printable_styled_string;
 use lib::core::rewrite::MergeConflictRemediation;
 use lib::git::Repo;
 use lib::git::RepoError;
 use lib::util::ExitCode;
+use tracing::level_filters::LevelFilter;
 use tracing_chrome::ChromeLayerBuilder;
 use tracing_error::ErrorLayer;
 use tracing_subscriber::fmt as tracing_fmt;
 use tracing_subscriber::prelude::*;
 use tracing_subscriber::EnvFilter;
 
-use crate::opts::ColorSetting;
 use crate::opts::Command;
 use crate::opts::Opts;
+use crate::opts::ResolveRevsetOptions;
+use crate::opts::Revset;
 use crate::opts::SnapshotSubcommand;
 use crate::opts::WrappedCommand;
+use crate::opts::{ColorSetting, TestSubcommand};
 use lib::core::config::env_vars::get_path_to_git;
 use lib::core::effects::Effects;
 use lib::core::formatting::Glyphs;
@@ -56,6 +59,7 @@ use lib::git::NonZeroOid;
 
 use self::reword::InitialCommitMessages;
 use self::smartlog::SmartlogOptions;
+use self::test::RawTestOptions;
 
 fn rewrite_args(args: Vec<OsString>) -> Vec<OsString> {
     let first_arg = match args.first() {
@@ -93,8 +97,6 @@ fn rewrite_args(args: Vec<OsString>) -> Vec<OsString> {
 /// Wrapper function for `main` to ensure that `Drop` is called for local
 /// variables, since `std::process::exit` will skip them.
 fn do_main_and_drop_locals() -> eyre::Result<i32> {
-    let _tracing_guard = install_tracing();
-
     let args = rewrite_args(std::env::args_os().collect_vec());
     let Opts {
         working_directory,
@@ -125,18 +127,25 @@ fn do_main_and_drop_locals() -> eyre::Result<i32> {
     };
     let effects = Effects::new(color);
 
+    let _tracing_guard = install_tracing(effects.clone());
+
     if let Some(ExitCode(exit_code)) = check_unsupported_config_options(&effects)? {
         let exit_code: i32 = exit_code.try_into()?;
         return Ok(exit_code);
     }
 
     let ExitCode(exit_code) = match command {
-        Command::Amend { move_options } => amend::amend(&effects, &git_run_info, &move_options)?,
+        Command::Amend { move_options } => amend::amend(
+            &effects,
+            &git_run_info,
+            &ResolveRevsetOptions::default(),
+            &move_options,
+        )?,
 
         Command::BugReport => bug_report::bug_report(&effects, &git_run_info)?,
 
-        Command::Checkout { checkout_options } => {
-            navigation::checkout(&effects, &git_run_info, &checkout_options)?
+        Command::Switch { switch_options } => {
+            navigation::switch(&effects, &git_run_info, &switch_options)?
         }
 
         Command::Gc | Command::HookPreAutoGc => {
@@ -146,9 +155,17 @@ fn do_main_and_drop_locals() -> eyre::Result<i32> {
 
         Command::Hide {
             revsets,
+            resolve_revset_options,
             delete_branches,
             recursive,
-        } => hide::hide(&effects, &git_run_info, revsets, delete_branches, recursive)?,
+        } => hide::hide(
+            &effects,
+            &git_run_info,
+            revsets,
+            &resolve_revset_options,
+            delete_branches,
+            recursive,
+        )?,
 
         Command::HookDetectEmptyCommit { old_commit_oid } => {
             let old_commit_oid: NonZeroOid = old_commit_oid.parse()?;
@@ -222,8 +239,9 @@ fn do_main_and_drop_locals() -> eyre::Result<i32> {
             dest,
             base,
             exact,
-            insert,
+            resolve_revset_options,
             move_options,
+            insert,
         } => r#move::r#move(
             &effects,
             &git_run_info,
@@ -231,8 +249,9 @@ fn do_main_and_drop_locals() -> eyre::Result<i32> {
             dest,
             base,
             exact,
-            insert,
+            &resolve_revset_options,
             &move_options,
+            insert,
         )?,
 
         Command::Next {
@@ -255,19 +274,29 @@ fn do_main_and_drop_locals() -> eyre::Result<i32> {
 
         Command::Query {
             revset,
+            resolve_revset_options,
             show_branches,
             raw,
-        } => query::query(&effects, &git_run_info, revset, show_branches, raw)?,
+        } => query::query(
+            &effects,
+            &git_run_info,
+            revset,
+            &resolve_revset_options,
+            show_branches,
+            raw,
+        )?,
 
         Command::Repair { dry_run } => repair::repair(&effects, dry_run)?,
 
         Command::Restack {
-            commits: revsets,
+            revsets,
+            resolve_revset_options,
             move_options,
         } => restack::restack(
             &effects,
             &git_run_info,
             revsets,
+            &resolve_revset_options,
             &move_options,
             MergeConflictRemediation::Retry,
         )?,
@@ -280,6 +309,7 @@ fn do_main_and_drop_locals() -> eyre::Result<i32> {
 
         Command::Reword {
             revsets,
+            resolve_revset_options,
             messages,
             force_rewrite_public_commits,
             discard,
@@ -295,6 +325,7 @@ fn do_main_and_drop_locals() -> eyre::Result<i32> {
             reword::reword(
                 &effects,
                 revsets,
+                &resolve_revset_options,
                 messages,
                 &git_run_info,
                 force_rewrite_public_commits,
@@ -302,16 +333,16 @@ fn do_main_and_drop_locals() -> eyre::Result<i32> {
         }
 
         Command::Smartlog {
-            show_hidden_commits,
             event_id,
             revset,
+            resolve_revset_options,
         } => smartlog::smartlog(
             &effects,
             &git_run_info,
             &SmartlogOptions {
-                show_hidden_commits,
                 event_id,
-                revset,
+                revset: revset.unwrap_or_else(Revset::default_smartlog_revset),
+                resolve_revset_options,
             },
         )?,
 
@@ -322,21 +353,89 @@ fn do_main_and_drop_locals() -> eyre::Result<i32> {
             }
         },
 
-        Command::Submit { create, revset } => {
-            submit::submit(&effects, &git_run_info, revset, create)?
-        }
+        Command::Submit {
+            create,
+            revset,
+            resolve_revset_options,
+        } => submit::submit(
+            &effects,
+            &git_run_info,
+            revset,
+            &resolve_revset_options,
+            create,
+        )?,
 
         Command::Sync {
-            update_refs,
+            pull,
             move_options,
             revsets,
-        } => sync::sync(&effects, &git_run_info, update_refs, &move_options, revsets)?,
+            resolve_revset_options,
+        } => sync::sync(
+            &effects,
+            &git_run_info,
+            pull,
+            &move_options,
+            revsets,
+            &resolve_revset_options,
+        )?,
+
+        Command::Test { subcommand } => match subcommand {
+            TestSubcommand::Run {
+                exec: command,
+                command: command_alias,
+                revset,
+                resolve_revset_options,
+                verbosity,
+                strategy,
+                jobs,
+            } => test::run(
+                &effects,
+                &git_run_info,
+                &RawTestOptions {
+                    exec: command,
+                    command: command_alias,
+                    strategy,
+                    jobs,
+                    verbosity: test::Verbosity::from(verbosity),
+                },
+                revset,
+                &resolve_revset_options,
+            )?,
+
+            TestSubcommand::Show {
+                exec: command,
+                command: command_alias,
+                revset,
+                resolve_revset_options,
+                verbosity,
+            } => test::show(
+                &effects,
+                &RawTestOptions {
+                    exec: command,
+                    command: command_alias,
+                    strategy: None,
+                    jobs: None,
+                    verbosity: test::Verbosity::from(verbosity),
+                },
+                revset,
+                &resolve_revset_options,
+            )?,
+
+            TestSubcommand::Clean {
+                revset,
+                resolve_revset_options,
+            } => test::clean(&effects, revset, &resolve_revset_options)?,
+        },
 
         Command::Undo { interactive, yes } => {
             undo::undo(&effects, &git_run_info, interactive, yes)?
         }
 
-        Command::Unhide { revsets, recursive } => hide::unhide(&effects, revsets, recursive)?,
+        Command::Unhide {
+            revsets,
+            resolve_revset_options,
+            recursive,
+        } => hide::unhide(&effects, revsets, &resolve_revset_options, recursive)?,
 
         Command::Wrap {
             git_executable: explicit_git_executable,
@@ -367,27 +466,11 @@ pub fn main() {
 }
 
 #[must_use = "This function returns a guard object to flush traces. Dropping it immediately is probably incorrect. Make sure that the returned value lives until tracing has finished."]
-fn install_tracing() -> eyre::Result<impl Drop> {
-    let (filter_layer, fmt_layer) = match EnvFilter::try_from_default_env() {
-        Ok(filter_layer) => {
-            let fmt_layer = tracing_fmt::layer()
-                .with_span_events(tracing_fmt::format::FmtSpan::CLOSE)
-                .with_target(false);
-            (Some(filter_layer), Some(fmt_layer))
-        }
-        Err(_) => {
-            // We would like the filter layer to apply *only* to the formatting
-            // layer. That way, the logging output is suppressed, but we still
-            // get spantraces for use with `color-eyre`. However, it's currently
-            // not possible (?), at least not without writing some a custom
-            // subscriber. See https://github.com/tokio-rs/tracing/pull/1523
-            //
-            // The workaround is to only display logging messages if `RUST_LOG`
-            // is set (which is unfortunate, because we'll miss out on
-            // `WARN`-level messages by default).
-            (None, None)
-        }
-    };
+fn install_tracing(effects: Effects) -> eyre::Result<impl Drop> {
+    let env_filter = EnvFilter::builder()
+        .with_default_directive(LevelFilter::WARN.into())
+        .from_env_lossy();
+    let fmt_layer = tracing_fmt::layer().with_writer(move || effects.clone().get_error_stream());
 
     let (profile_layer, flush_guard): (_, Box<dyn Any>) = {
         // We may invoke a hook that calls back into `git-branchless`. In that case,
@@ -436,8 +519,7 @@ fn install_tracing() -> eyre::Result<impl Drop> {
 
     tracing_subscriber::registry()
         .with(ErrorLayer::default())
-        .with(filter_layer)
-        .with(fmt_layer)
+        .with(fmt_layer.with_filter(env_filter))
         .with(profile_layer)
         .try_init()?;
 
@@ -462,16 +544,13 @@ Here are some options:
 - To unset the configuration option, run: git config --unset extensions.worktreeConfig
   - This is safe unless you created another worktree also using a sparse checkout.
 - Try upgrading to Git v2.36+ and reinitializing your sparse checkout.",
-                error = printable_styled_string(
-                    effects.get_glyphs(),
-                    StyledString::styled(
-                        "\
+                error = effects.get_glyphs().render(StyledString::styled(
+                    "\
 Error: the Git configuration setting `extensions.worktreeConfig` is enabled in
 this repository. Due to upstream libgit2 limitations, git-branchless does not
 support repositories with this configuration option enabled.",
-                        BaseColor::Red.light()
-                    )
-                )?,
+                    BaseColor::Red.light()
+                ))?,
             )?;
             return Ok(Some(ExitCode(1)));
         }
