@@ -8,17 +8,17 @@ use std::convert::TryFrom;
 use std::fmt::Write;
 use std::time::SystemTime;
 
-use console::style;
-use eden_dag::DagAlgorithm;
+use eden_dag::{DagAlgorithm, VertexName};
 use lib::core::repo_ext::RepoExt;
 use lib::util::ExitCode;
 use rayon::ThreadPoolBuilder;
 use tracing::instrument;
 
-use crate::opts::{MoveOptions, Revset};
+use crate::opts::{MoveOptions, ResolveRevsetOptions, Revset};
 use crate::revset::resolve_commits;
 use lib::core::config::{
-    get_hint_enabled, get_restack_preserve_timestamps, print_hint_suppression_notice, Hint,
+    get_hint_enabled, get_hint_string, get_restack_preserve_timestamps,
+    print_hint_suppression_notice, Hint,
 };
 use lib::core::dag::{commit_set_to_vec, sorted_commit_set, union_all, CommitSet, Dag};
 use lib::core::effects::Effects;
@@ -32,7 +32,7 @@ use lib::git::{GitRunInfo, NonZeroOid, Repo};
 #[instrument]
 fn resolve_base_commit(
     dag: &Dag,
-    merge_base_oid: Option<NonZeroOid>,
+    merge_base_oid: Option<VertexName>,
     oid: NonZeroOid,
 ) -> eyre::Result<NonZeroOid> {
     let bases = match merge_base_oid {
@@ -64,8 +64,9 @@ pub fn r#move(
     dest: Option<Revset>,
     bases: Vec<Revset>,
     exacts: Vec<Revset>,
-    insert: bool,
+    resolve_revset_options: &ResolveRevsetOptions,
     move_options: &MoveOptions,
+    insert: bool,
 ) -> eyre::Result<ExitCode> {
     let sources_provided = !sources.is_empty();
     let bases_provided = !bases.is_empty();
@@ -100,21 +101,29 @@ pub fn r#move(
         &references_snapshot,
     )?;
 
-    let source_oids: CommitSet = match resolve_commits(effects, &repo, &mut dag, sources) {
-        Ok(commit_sets) => union_all(&commit_sets),
-        Err(err) => {
-            err.describe(effects)?;
-            return Ok(ExitCode(1));
-        }
-    };
-    let base_oids: CommitSet = match resolve_commits(effects, &repo, &mut dag, bases) {
-        Ok(commit_sets) => union_all(&commit_sets),
-        Err(err) => {
-            err.describe(effects)?;
-            return Ok(ExitCode(1));
-        }
-    };
-    let exact_components = match resolve_commits(effects, &repo, &mut dag, exacts) {
+    let source_oids: CommitSet =
+        match resolve_commits(effects, &repo, &mut dag, &sources, resolve_revset_options) {
+            Ok(commit_sets) => union_all(&commit_sets),
+            Err(err) => {
+                err.describe(effects)?;
+                return Ok(ExitCode(1));
+            }
+        };
+    let base_oids: CommitSet =
+        match resolve_commits(effects, &repo, &mut dag, &bases, resolve_revset_options) {
+            Ok(commit_sets) => union_all(&commit_sets),
+            Err(err) => {
+                err.describe(effects)?;
+                return Ok(ExitCode(1));
+            }
+        };
+    let exact_components = match resolve_commits(
+        effects,
+        &repo,
+        &mut dag,
+        &exacts,
+        resolve_revset_options,
+    ) {
         Ok(commit_sets) => {
             let exact_oids = union_all(&commit_sets);
             let mut components: HashMap<NonZeroOid, CommitSet> = HashMap::new();
@@ -158,7 +167,13 @@ pub fn r#move(
         }
     };
 
-    let dest_oid: NonZeroOid = match resolve_commits(effects, &repo, &mut dag, vec![dest.clone()]) {
+    let dest_oid: NonZeroOid = match resolve_commits(
+        effects,
+        &repo,
+        &mut dag,
+        &[dest.clone()],
+        resolve_revset_options,
+    ) {
         Ok(commit_sets) => match commit_set_to_vec(&commit_sets[0])?.as_slice() {
             [only_commit_oid] => *only_commit_oid,
             other => {
@@ -192,7 +207,9 @@ pub fn r#move(
     let base_oids = {
         let mut result = Vec::new();
         for base_oid in commit_set_to_vec(&base_oids)? {
-            let merge_base_oid = dag.get_one_merge_base_oid(effects, &repo, base_oid, dest_oid)?;
+            let merge_base_oid = dag
+                .query()
+                .gca_one(vec![base_oid, dest_oid].into_iter().collect::<CommitSet>())?;
             let base_commit_oid = resolve_base_commit(&dag, merge_base_oid, base_oid)?;
             result.push(CommitSet::from(base_commit_oid))
         }
@@ -210,7 +227,7 @@ pub fn r#move(
                 writeln!(
                     effects.get_output_stream(),
                     "{}: you can omit the --base flag in this case, as it defaults to HEAD",
-                    style("hint").blue().bold()
+                    effects.get_glyphs().render(get_hint_string())?,
                 )?;
             }
 
@@ -219,7 +236,7 @@ pub fn r#move(
                 writeln!(
                     effects.get_output_stream(),
                     "{}: you can omit the --dest flag in this case, as it defaults to HEAD",
-                    style("hint").blue().bold()
+                    effects.get_glyphs().render(get_hint_string())?,
                 )?;
             }
 
@@ -362,8 +379,8 @@ pub fn r#move(
             let component_children: CommitSet = dag
                 .query()
                 .children(component.clone())?
-                .difference(component)
-                .difference(&dag.obsolete_commits);
+                .difference(component);
+            let component_children = dag.filter_visible_commits(component_children)?;
 
             for component_child in commit_set_to_vec(&component_children)? {
                 // If the range being extracted has any child commits, then we
@@ -442,8 +459,8 @@ pub fn r#move(
                 .query()
                 .children(CommitSet::from(dest_oid))?
                 .difference(&source_oids)
-                .difference(&exact_oids)
-                .difference(&dag.obsolete_commits);
+                .difference(&exact_oids);
+            let dest_children = dag.filter_visible_commits(dest_children)?;
 
             for dest_child in commit_set_to_vec(&dest_children)? {
                 builder.move_subtree(dest_child, vec![source_head])?;
