@@ -15,13 +15,14 @@ use lib::util::ExitCode;
 use tracing::{instrument, warn};
 
 use crate::commands::smartlog::make_smartlog_graph;
-use crate::opts::{CheckoutOptions, TraverseCommitsOptions};
+use crate::opts::{SwitchOptions, TraverseCommitsOptions};
+use crate::revset::resolve_default_smartlog_commits;
 use crate::tui::prompt_select_commit;
 use lib::core::config::get_next_interactive;
 use lib::core::dag::{sorted_commit_set, CommitSet, Dag};
 use lib::core::effects::Effects;
 use lib::core::eventlog::{EventLogDb, EventReplayer};
-use lib::core::formatting::{printable_styled_string, Pluralize};
+use lib::core::formatting::Pluralize;
 use lib::core::node_descriptors::{
     BranchesDescriptor, CommitMessageDescriptor, CommitOidDescriptor,
     DifferentialRevisionDescriptor, NodeDescriptor, Redactor, RelativeTimeDescriptor,
@@ -104,10 +105,8 @@ fn advance(
         let candidate_commits = match command {
             Command::Next => {
                 let child_commits = || -> eyre::Result<CommitSet> {
-                    let result = dag
-                        .query()
-                        .children(CommitSet::from(current_oid))?
-                        .difference(&dag.obsolete_commits);
+                    let result = dag.query().children(CommitSet::from(current_oid))?;
+                    let result = dag.filter_visible_commits(result)?;
                     Ok(result)
                 };
 
@@ -231,16 +230,13 @@ fn advance(
                 writeln!(
                     effects.get_output_stream(),
                     "{}",
-                    printable_styled_string(
-                        glyphs,
-                        StyledString::styled(
-                            format!(
-                                "No more {} commits to go to after traversing {}.",
-                                pluralize.unit.0, pluralize,
-                            ),
-                            BaseColor::Yellow.light()
-                        )
-                    )?
+                    glyphs.render(StyledString::styled(
+                        format!(
+                            "No more {} commits to go to after traversing {}.",
+                            pluralize.unit.0, pluralize,
+                        ),
+                        BaseColor::Yellow.light()
+                    ))?
                 )?;
 
                 if i == 0 {
@@ -284,7 +280,7 @@ fn advance(
                         effects.get_output_stream(),
                         "  {} {}{}",
                         glyphs.bullet_point,
-                        printable_styled_string(glyphs, child.friendly_describe(glyphs)?)?,
+                        glyphs.render(child.friendly_describe(glyphs)?)?,
                         descriptor
                     )?;
                 }
@@ -460,9 +456,9 @@ pub fn traverse_commits(
     )
 }
 
-fn get_initial_query(checkout_options: &CheckoutOptions) -> Option<&str> {
+fn get_initial_query(checkout_options: &SwitchOptions) -> Option<&str> {
     match checkout_options {
-        CheckoutOptions {
+        SwitchOptions {
             interactive: true,
             target: Some(target),
             branch_name: _,
@@ -470,7 +466,7 @@ fn get_initial_query(checkout_options: &CheckoutOptions) -> Option<&str> {
             merge: _,
         } => Some(target),
 
-        CheckoutOptions {
+        SwitchOptions {
             interactive: false,
             target: Some(_),
             branch_name: None,
@@ -478,14 +474,14 @@ fn get_initial_query(checkout_options: &CheckoutOptions) -> Option<&str> {
             merge: false,
         } => None,
 
-        CheckoutOptions {
+        SwitchOptions {
             interactive: true,
             target: None,
             branch_name: _,
             force: _,
             merge: _,
         }
-        | CheckoutOptions {
+        | SwitchOptions {
             interactive: false,
             target: None,
             branch_name: None,
@@ -493,28 +489,28 @@ fn get_initial_query(checkout_options: &CheckoutOptions) -> Option<&str> {
             merge: false,
         } => Some(""),
 
-        CheckoutOptions {
+        SwitchOptions {
             interactive: false,
             target: Some(_),
             branch_name: _,
             force: _,
             merge: _,
         }
-        | CheckoutOptions {
+        | SwitchOptions {
             interactive: false,
             target: _,
             branch_name: Some(_),
             force: _,
             merge: _,
         }
-        | CheckoutOptions {
+        | SwitchOptions {
             interactive: false,
             target: _,
             branch_name: _,
             force: true,
             merge: _,
         }
-        | CheckoutOptions {
+        | SwitchOptions {
             interactive: false,
             target: _,
             branch_name: _,
@@ -524,19 +520,19 @@ fn get_initial_query(checkout_options: &CheckoutOptions) -> Option<&str> {
     }
 }
 
-/// Interactively checkout a commit from the smartlog.
-pub fn checkout(
+/// Interactively switch to a commit from the smartlog.
+pub fn switch(
     effects: &Effects,
     git_run_info: &GitRunInfo,
-    checkout_options: &CheckoutOptions,
+    switch_options: &SwitchOptions,
 ) -> eyre::Result<ExitCode> {
-    let CheckoutOptions {
+    let SwitchOptions {
         interactive: _,
         branch_name,
         force,
         merge,
         target,
-    } = checkout_options;
+    } = switch_options;
 
     let now = SystemTime::now();
     let repo = Repo::from_current_dir()?;
@@ -547,7 +543,7 @@ pub fn checkout(
     let event_tx_id = event_log_db.make_transaction_id(now, "checkout")?;
     let event_replayer = EventReplayer::from_event_log_db(effects, &repo, &event_log_db)?;
     let event_cursor = event_replayer.make_default_cursor();
-    let dag = Dag::open_and_sync(
+    let mut dag = Dag::open_and_sync(
         effects,
         &repo,
         &event_replayer,
@@ -555,17 +551,17 @@ pub fn checkout(
         &references_snapshot,
     )?;
 
+    let commits = resolve_default_smartlog_commits(effects, &repo, &mut dag)?;
     let graph = make_smartlog_graph(
         effects,
         &repo,
         &dag,
         &event_replayer,
         event_cursor,
-        &dag.observed_commits,
-        true,
+        &commits,
     )?;
 
-    let initial_query = get_initial_query(checkout_options);
+    let initial_query = get_initial_query(switch_options);
     let target: Option<CheckoutTarget> = match initial_query {
         None => target.clone().map(CheckoutTarget::Unknown),
         Some(initial_query) => {
