@@ -5,13 +5,12 @@ use itertools::Itertools;
 use lib::core::dag::{commit_set_to_vec, Dag};
 use lib::core::effects::{Effects, OperationType};
 use lib::core::eventlog::{EventLogDb, EventReplayer};
-use lib::core::formatting::printable_styled_string;
 use lib::core::repo_ext::RepoExt;
 use lib::git::{CategorizedReferenceName, GitRunInfo, Repo};
 use lib::util::ExitCode;
 use tracing::instrument;
 
-use crate::opts::Revset;
+use crate::opts::{ResolveRevsetOptions, Revset};
 use crate::revset::resolve_commits;
 
 #[instrument]
@@ -19,6 +18,7 @@ pub fn query(
     effects: &Effects,
     git_run_info: &GitRunInfo,
     query: Revset,
+    resolve_revset_options: &ResolveRevsetOptions,
     show_branches: bool,
     raw: bool,
 ) -> eyre::Result<ExitCode> {
@@ -36,13 +36,14 @@ pub fn query(
         &references_snapshot,
     )?;
 
-    let commit_set = match resolve_commits(effects, &repo, &mut dag, vec![query]) {
-        Ok(commit_sets) => commit_sets[0].clone(),
-        Err(err) => {
-            err.describe(effects)?;
-            return Ok(ExitCode(1));
-        }
-    };
+    let commit_set =
+        match resolve_commits(effects, &repo, &mut dag, &[query], resolve_revset_options) {
+            Ok(commit_sets) => commit_sets[0].clone(),
+            Err(err) => {
+                err.describe(effects)?;
+                return Ok(ExitCode(1));
+            }
+        };
 
     if show_branches {
         let commit_oids = {
@@ -83,10 +84,9 @@ pub fn query(
                 writeln!(
                     effects.get_output_stream(),
                     "{}",
-                    printable_styled_string(
-                        effects.get_glyphs(),
-                        commit.friendly_describe(effects.get_glyphs())?
-                    )?,
+                    effects
+                        .get_glyphs()
+                        .render(commit.friendly_describe(effects.get_glyphs())?)?,
                 )?;
             }
         }
