@@ -8,7 +8,6 @@ use std::collections::{HashMap, HashSet};
 use std::convert::TryFrom;
 use std::fmt::Write;
 use std::fs::File;
-use std::io::Write as OtherWrite;
 use std::time::SystemTime;
 
 use bstr::{ByteSlice, ByteVec};
@@ -24,7 +23,7 @@ use lib::core::config::{
 use lib::core::dag::{sorted_commit_set, union_all, CommitSet, Dag};
 use lib::core::effects::Effects;
 use lib::core::eventlog::{EventLogDb, EventReplayer};
-use lib::core::formatting::{printable_styled_string, Glyphs, Pluralize};
+use lib::core::formatting::{Glyphs, Pluralize};
 use lib::core::node_descriptors::{render_node_descriptors, CommitOidDescriptor, NodeObject};
 use lib::core::rewrite::{
     execute_rebase_plan, BuildRebasePlanOptions, ExecuteRebasePlanOptions, ExecuteRebasePlanResult,
@@ -32,7 +31,7 @@ use lib::core::rewrite::{
 };
 use lib::git::{message_prettify, Commit, GitRunInfo, MaybeZeroOid, NonZeroOid, Repo};
 
-use crate::opts::Revset;
+use crate::opts::{ResolveRevsetOptions, Revset};
 use crate::revset::resolve_commits;
 
 /// The commit message(s) provided by the user.
@@ -53,6 +52,7 @@ pub enum InitialCommitMessages {
 pub fn reword(
     effects: &Effects,
     revsets: Vec<Revset>,
+    resolve_revset_options: &ResolveRevsetOptions,
     messages: InitialCommitMessages,
     git_run_info: &GitRunInfo,
     force_rewrite_public_commits: bool,
@@ -71,7 +71,13 @@ pub fn reword(
         &references_snapshot,
     )?;
 
-    let commits = match resolve_commits_from_hashes(&repo, &mut dag, effects, revsets)? {
+    let commits = match resolve_commits_from_hashes(
+        &repo,
+        &mut dag,
+        effects,
+        revsets,
+        resolve_revset_options,
+    )? {
         Some(commits) => commits,
         None => return Ok(ExitCode(1)),
     };
@@ -96,9 +102,14 @@ pub fn reword(
     let messages = match messages {
         InitialCommitMessages::Discard | InitialCommitMessages::Messages(_) => messages,
         InitialCommitMessages::FixUp(revset) => {
-            let commits_to_fixup =
-                resolve_commits_from_hashes(&repo, &mut dag, effects, vec![revset.clone()])?
-                    .unwrap_or_default();
+            let commits_to_fixup = resolve_commits_from_hashes(
+                &repo,
+                &mut dag,
+                effects,
+                vec![revset.clone()],
+                resolve_revset_options,
+            )?
+            .unwrap_or_default();
             let commit_to_fixup = match commits_to_fixup.as_slice() {
                 [commit_to_fixup] => {
                     let commits: CommitSet = commits.iter().map(|c| c.get_oid()).collect();
@@ -307,6 +318,7 @@ fn resolve_commits_from_hashes<'repo>(
     dag: &mut Dag,
     effects: &Effects,
     revsets: Vec<Revset>,
+    resolve_revset_options: &ResolveRevsetOptions,
 ) -> eyre::Result<Option<Vec<Commit<'repo>>>> {
     let hashes = if revsets.is_empty() {
         vec![Revset("HEAD".to_string())]
@@ -314,7 +326,7 @@ fn resolve_commits_from_hashes<'repo>(
         revsets
     };
 
-    let commit_sets = match resolve_commits(effects, repo, dag, hashes) {
+    let commit_sets = match resolve_commits(effects, repo, dag, &hashes, resolve_revset_options) {
         Ok(commit_sets) => commit_sets,
         Err(err) => {
             err.describe(effects)?;
@@ -500,6 +512,7 @@ fn prepare_messages(
 
         let mut w = File::create(repo.get_path().join("REWORD_EDITMSG"))
             .context("Creating REWORD_EDITMSG file")?;
+        use std::io::Write;
         writeln!(
             &mut w,
             "{} This file was created by `git branchless reword` at {}\n\
@@ -656,8 +669,7 @@ fn render_status_report(
         writeln!(
             effects.get_output_stream(),
             "Reworded commit {} as {}",
-            printable_styled_string(
-                &glyphs,
+            glyphs.render(
                 // Commit doesn't offer `friendly_describe_oid`, so we'll do it ourselves
                 render_node_descriptors(
                     &glyphs,
@@ -667,7 +679,7 @@ fn render_status_report(
                     &mut [&mut CommitOidDescriptor::new(true)?],
                 )?
             )?,
-            printable_styled_string(&glyphs, replacement_commit.friendly_describe(&glyphs)?)?
+            glyphs.render(replacement_commit.friendly_describe(&glyphs)?)?
         )?;
     }
 
@@ -718,7 +730,7 @@ mod tests {
         }
 
         git.run(&["config", "commit.template", "template.txt"])?;
-        git.write_file(
+        git.write_file_txt(
             "template",
             "\
 This is a template!
