@@ -5,12 +5,10 @@
 
 use std::cmp::Ordering;
 use std::fmt::Write;
-use std::mem::swap;
 use std::time::SystemTime;
 
-use console::style;
 use eden_dag::DagAlgorithm;
-use lib::core::config::{get_hint_enabled, print_hint_suppression_notice, Hint};
+use lib::core::config::{get_hint_enabled, get_hint_string, print_hint_suppression_notice, Hint};
 use lib::core::repo_ext::RepoExt;
 use lib::core::rewrite::find_rewrite_target;
 use lib::util::ExitCode;
@@ -19,7 +17,7 @@ use tracing::instrument;
 use lib::core::dag::{CommitSet, Dag};
 use lib::core::effects::Effects;
 use lib::core::eventlog::{EventLogDb, EventReplayer};
-use lib::core::formatting::{printable_styled_string, Pluralize};
+use lib::core::formatting::Pluralize;
 use lib::core::node_descriptors::{
     BranchesDescriptor, CommitMessageDescriptor, CommitOidDescriptor,
     DifferentialRevisionDescriptor, ObsolescenceExplanationDescriptor, Redactor,
@@ -40,27 +38,45 @@ mod graph {
     use lib::core::gc::mark_commit_reachable;
     use tracing::instrument;
 
-    use lib::core::dag::{commit_set_to_vec, CommitSet, Dag};
+    use lib::core::dag::{commit_set_to_vec, CommitSet, CommitVertex, Dag};
     use lib::core::effects::{Effects, OperationType};
     use lib::core::eventlog::{EventCursor, EventReplayer};
     use lib::core::node_descriptors::NodeObject;
     use lib::git::{Commit, Time};
     use lib::git::{NonZeroOid, Repo};
 
+    #[derive(Debug)]
+    pub struct AncestorInfo {
+        pub oid: NonZeroOid,
+        pub distance: usize,
+    }
+
+    #[derive(Clone, Debug, PartialEq, Eq, Hash)]
+    pub struct ChildInfo {
+        pub oid: NonZeroOid,
+        pub is_merge_child: bool,
+    }
     /// Node contained in the smartlog commit graph.
     #[derive(Debug)]
     pub struct Node<'repo> {
         /// The underlying commit object.
         pub object: NodeObject<'repo>,
 
-        /// The OID of the parent node in the smartlog commit graph.
+        /// The OIDs of the parent nodes in the smartlog commit graph.
         ///
-        /// This is different from inspecting `commit.parents()`,& since the smartlog
+        /// This is different from inspecting `commit.parents()`, since the smartlog
         /// will hide most nodes from the commit graph, including parent nodes.
-        pub parent: Option<NonZeroOid>,
+        pub parents: Vec<NonZeroOid>,
 
         /// The OIDs of the children nodes in the smartlog commit graph.
-        pub children: Vec<NonZeroOid>,
+        pub children: Vec<ChildInfo>,
+
+        /// Information about a non-immediate, non-main branch ancestor node in
+        /// the smartlog commit graph.
+        pub ancestor_info: Option<AncestorInfo>,
+
+        /// The OIDs of any non-immediate descendant nodes in the smartlog commit graph.
+        pub descendants: Vec<ChildInfo>,
 
         /// Indicates that this is a commit to the main branch.
         ///
@@ -83,6 +99,13 @@ mod graph {
         /// where you commit directly to the main branch and then later rewrite the
         /// commit.
         pub is_obsolete: bool,
+
+        /// Indicates that this commit has descendants, but that none of them
+        /// are included in the graph.
+        ///
+        /// This allows us to indicate a "false head" to the user. Otherwise,
+        /// this commit would look like a normal, descendant-less head.
+        pub num_omitted_descendants: usize,
     }
 
     /// Graph of commits that the user is working on.
@@ -114,35 +137,27 @@ mod graph {
         }
     }
 
-    /// Find additional commits that should be displayed.
+    /// Build the smartlog graph by finding additional commits that should be displayed.
     ///
     /// For example, if you check out a commit that has intermediate parent commits
     /// between it and the main branch, those intermediate commits should be shown
     /// (or else you won't get a good idea of the line of development that happened
     /// for this commit since the main branch).
     #[instrument]
-    fn walk_from_active_heads<'repo>(
+    fn build_graph<'repo>(
         effects: &Effects,
         repo: &'repo Repo,
         dag: &Dag,
-        public_commits: &CommitSet,
-        active_heads: &CommitSet,
+        commits: &CommitSet,
     ) -> eyre::Result<SmartlogGraph<'repo>> {
         let mut graph: HashMap<NonZeroOid, Node> = {
             let mut result = HashMap::new();
-            for vertex in active_heads.iter()? {
-                let vertex = vertex?;
-                let path_to_main_branch =
-                    dag.find_path_to_main_branch(effects, CommitSet::from(vertex.clone()))?;
-                let path_to_main_branch = match path_to_main_branch {
-                    Some(path_to_main_branch) => path_to_main_branch,
-                    None => CommitSet::from(vertex.clone()),
-                };
-
-                for vertex in path_to_main_branch.iter_rev()? {
-                    let vertex = vertex?;
-                    let oid = NonZeroOid::try_from(vertex.clone())?;
+            for vertex in commit_set_to_vec(commits)? {
+                let vertex = CommitSet::from(vertex);
+                let merge_bases = dag.query().gca_all(dag.main_branch_commit.union(&vertex))?;
+                let vertices = vertex.union(&merge_bases);
 
+                for oid in commit_set_to_vec(&vertices)? {
                     let object = match repo.find_commit(oid)? {
                         Some(commit) => NodeObject::Commit { commit },
                         None => {
@@ -155,10 +170,13 @@ mod graph {
                         oid,
                         Node {
                             object,
-                            parent: None,         // populated below
+                            parents: Vec::new(),  // populated below
                             children: Vec::new(), // populated below
-                            is_main: public_commits.contains(&vertex)?,
-                            is_obsolete: dag.obsolete_commits.contains(&vertex)?,
+                            ancestor_info: None,
+                            descendants: Vec::new(), // populated below
+                            is_main: dag.is_public_commit(oid)?,
+                            is_obsolete: dag.query_obsolete_commits().contains(&oid.into())?,
+                            num_omitted_descendants: 0, // populated below
                         },
                     );
                 }
@@ -166,29 +184,118 @@ mod graph {
             result
         };
 
-        // Find immediate parent-child links.
-        let links: Vec<(NonZeroOid, NonZeroOid)> = {
-            let non_main_node_oids =
-                graph.iter().filter_map(
-                    |(child_oid, node)| if !node.is_main { Some(child_oid) } else { None },
-                );
+        let mut immediate_links: Vec<(NonZeroOid, NonZeroOid, bool)> = Vec::new();
+        let mut non_immediate_links: Vec<(NonZeroOid, NonZeroOid, bool)> = Vec::new();
 
-            let mut links = Vec::new();
-            for child_oid in non_main_node_oids {
-                let parent_vertexes = dag.query().parents(CommitSet::from(*child_oid))?;
-                let parent_oids = commit_set_to_vec(&parent_vertexes)?;
-                for parent_oid in parent_oids {
-                    if graph.contains_key(&parent_oid) {
-                        links.push((*child_oid, parent_oid))
+        let non_main_node_oids =
+            graph
+                .iter()
+                .filter_map(|(child_oid, node)| if !node.is_main { Some(child_oid) } else { None });
+
+        let graph_vertices: CommitSet = graph.keys().cloned().collect();
+        for child_oid in non_main_node_oids {
+            let parent_vertices = dag.query().parent_names(CommitVertex::from(*child_oid))?;
+
+            // Find immediate parent-child links.
+            match parent_vertices.as_slice() {
+                [] => {}
+                [first_parent_vertex, merge_parent_vertices @ ..] => {
+                    if graph_vertices.contains(first_parent_vertex)? {
+                        let first_parent_oid = NonZeroOid::try_from(first_parent_vertex.clone())?;
+                        immediate_links.push((*child_oid, first_parent_oid, false));
+                    }
+                    for merge_parent_vertex in merge_parent_vertices {
+                        if graph_vertices.contains(merge_parent_vertex)? {
+                            let merge_parent_oid =
+                                NonZeroOid::try_from(merge_parent_vertex.clone())?;
+                            immediate_links.push((*child_oid, merge_parent_oid, true));
+                        }
                     }
                 }
             }
-            links
-        };
 
-        for (child_oid, parent_oid) in links.iter() {
-            graph.get_mut(child_oid).unwrap().parent = Some(*parent_oid);
-            graph.get_mut(parent_oid).unwrap().children.push(*child_oid);
+            // Find non-immediate ancestor links.
+            for excluded_parent_vertex in parent_vertices {
+                if graph_vertices.contains(&excluded_parent_vertex)? {
+                    continue;
+                }
+
+                // Find the nearest ancestor that is included in the graph and
+                // also on the same branch.
+
+                let parent_set = CommitSet::from(excluded_parent_vertex);
+                let merge_base = dag
+                    .query()
+                    .gca_one(dag.main_branch_commit.union(&parent_set))?;
+
+                let path_to_main_branch = match merge_base {
+                    Some(merge_base) => {
+                        dag.query().range(CommitSet::from(merge_base), parent_set)?
+                    }
+                    None => CommitSet::empty(),
+                };
+                let nearest_branch_ancestor = dag
+                    .query()
+                    .heads_ancestors(path_to_main_branch.intersection(&graph_vertices))?;
+
+                let ancestor_oids = commit_set_to_vec(&nearest_branch_ancestor)?;
+                for ancestor_oid in ancestor_oids.iter() {
+                    non_immediate_links.push((*ancestor_oid, *child_oid, false));
+                }
+            }
+        }
+
+        for (child_oid, parent_oid, is_merge_link) in immediate_links.iter() {
+            graph.get_mut(child_oid).unwrap().parents.push(*parent_oid);
+            graph.get_mut(parent_oid).unwrap().children.push(ChildInfo {
+                oid: *child_oid,
+                is_merge_child: *is_merge_link,
+            });
+        }
+
+        for (ancestor_oid, descendent_oid, is_merge_link) in non_immediate_links.iter() {
+            let distance = dag
+                .query()
+                .range(
+                    CommitSet::from(*ancestor_oid),
+                    CommitSet::from(*descendent_oid),
+                )?
+                .difference(&vec![*ancestor_oid, *descendent_oid].into_iter().collect())
+                .count()?;
+            graph.get_mut(descendent_oid).unwrap().ancestor_info = Some(AncestorInfo {
+                oid: *ancestor_oid,
+                distance,
+            });
+            graph
+                .get_mut(ancestor_oid)
+                .unwrap()
+                .descendants
+                .push(ChildInfo {
+                    oid: *descendent_oid,
+                    is_merge_child: *is_merge_link,
+                })
+        }
+
+        for (oid, node) in graph.iter_mut() {
+            let oid_set = CommitSet::from(*oid);
+            let is_main_head = !dag.main_branch_commit.intersection(&oid_set).is_empty()?;
+            let ancestor_of_main = node.is_main && !is_main_head;
+            let has_descendants_in_graph =
+                !node.children.is_empty() || !node.descendants.is_empty();
+
+            if ancestor_of_main || has_descendants_in_graph {
+                continue;
+            }
+
+            // This node has no descendants in the graph, so it's a
+            // false head if it has *any* visible descendants.
+            let descendants_not_in_graph = dag
+                .query()
+                .descendants(oid_set.clone())?
+                .difference(&oid_set);
+            let descendants_not_in_graph = dag.filter_visible_commits(descendants_not_in_graph)?;
+
+            node.num_omitted_descendants = descendants_not_in_graph.count()?;
         }
 
         Ok(SmartlogGraph { nodes: graph })
@@ -211,8 +318,12 @@ mod graph {
             })
             .collect();
         for node in graph.nodes.values_mut() {
-            node.children
-                .sort_by_key(|child_oid| (&commit_times[child_oid], child_oid.to_string()));
+            node.children.sort_by_key(
+                |ChildInfo {
+                     oid,
+                     is_merge_child,
+                 }| (&commit_times[oid], *is_merge_child, oid.to_string()),
+            );
         }
     }
 
@@ -224,28 +335,23 @@ mod graph {
         dag: &Dag,
         event_replayer: &EventReplayer,
         event_cursor: EventCursor,
-        observed_commits: &CommitSet,
-        remove_commits: bool,
+        commits: &CommitSet,
     ) -> eyre::Result<SmartlogGraph<'repo>> {
         let (effects, _progress) = effects.start_operation(OperationType::MakeGraph);
 
         let mut graph = {
             let (effects, _progress) = effects.start_operation(OperationType::WalkCommits);
 
-            let public_commits = dag.query_public_commits()?;
+            // HEAD and main head must be included
+            let commits = commits
+                .union(&dag.head_commit)
+                .union(&dag.main_branch_commit);
 
-            let observed_commits = if remove_commits {
-                observed_commits.difference(&dag.obsolete_commits)
-            } else {
-                observed_commits.clone()
-            };
-
-            let active_heads = dag.query_active_heads(&public_commits, &observed_commits)?;
-            for oid in commit_set_to_vec(&active_heads)? {
+            for oid in commit_set_to_vec(&commits)? {
                 mark_commit_reachable(repo, oid)?;
             }
 
-            walk_from_active_heads(&effects, repo, dag, &public_commits, &active_heads)?
+            build_graph(&effects, repo, dag, &commits)?
         };
         sort_children(&mut graph);
         Ok(graph)
@@ -254,22 +360,24 @@ mod graph {
 
 mod render {
     use std::cmp::Ordering;
+    use std::collections::HashSet;
+    use std::convert::TryFrom;
 
-    use cursive::theme::Effect;
+    use cursive::theme::{BaseColor, Effect};
     use cursive::utils::markup::StyledString;
     use eden_dag::DagAlgorithm;
     use tracing::instrument;
 
     use lib::core::dag::{CommitSet, CommitVertex, Dag};
     use lib::core::effects::Effects;
-    use lib::core::formatting::set_effect;
+    use lib::core::formatting::{set_effect, Pluralize};
     use lib::core::formatting::{Glyphs, StyledStringBuilder};
     use lib::core::node_descriptors::{render_node_descriptors, NodeDescriptor};
     use lib::git::{NonZeroOid, Repo};
 
-    use crate::opts::Revset;
+    use crate::opts::{ResolveRevsetOptions, Revset};
 
-    use super::graph::SmartlogGraph;
+    use super::graph::{AncestorInfo, ChildInfo, SmartlogGraph};
 
     /// Split fully-independent subgraphs into multiple graphs.
     ///
@@ -279,7 +387,6 @@ mod render {
     /// Returns the list such that the topologically-earlier subgraphs are first in
     /// the list (i.e. those that would be rendered at the bottom of the smartlog).
     fn split_commit_graph_by_roots(
-        effects: &Effects,
         repo: &Repo,
         dag: &Dag,
         graph: &SmartlogGraph,
@@ -287,7 +394,7 @@ mod render {
         let mut root_commit_oids: Vec<NonZeroOid> = graph
             .nodes
             .iter()
-            .filter(|(_oid, node)| node.parent.is_none())
+            .filter(|(_oid, node)| node.parents.is_empty() && node.ancestor_info.is_none())
             .map(|(oid, _node)| oid)
             .copied()
             .collect();
@@ -301,10 +408,13 @@ mod render {
                 _ => return lhs_oid.cmp(rhs_oid),
             };
 
-            let merge_base_oid = dag.get_one_merge_base_oid(effects, repo, *lhs_oid, *rhs_oid);
+            let merge_base_oid = dag
+                .query()
+                .gca_one(vec![*lhs_oid, *rhs_oid].into_iter().collect::<CommitSet>());
             let merge_base_oid = match merge_base_oid {
                 Err(_) => return lhs_oid.cmp(rhs_oid),
-                Ok(merge_base_oid) => merge_base_oid,
+                Ok(None) => None,
+                Ok(Some(merge_base_oid)) => NonZeroOid::try_from(merge_base_oid).ok(),
             };
 
             match merge_base_oid {
@@ -339,60 +449,162 @@ mod render {
         let current_node = &graph.nodes[&current_oid];
         let is_head = Some(current_oid) == head_oid;
 
-        let text = render_node_descriptors(glyphs, &current_node.object, commit_descriptors)?;
-        let cursor = match (current_node.is_main, current_node.is_obsolete, is_head) {
-            (false, false, false) => glyphs.commit_visible,
-            (false, false, true) => glyphs.commit_visible_head,
-            (false, true, false) => glyphs.commit_obsolete,
-            (false, true, true) => glyphs.commit_obsolete_head,
-            (true, false, false) => glyphs.commit_main,
-            (true, false, true) => glyphs.commit_main_head,
-            (true, true, false) => glyphs.commit_main_obsolete,
-            (true, true, true) => glyphs.commit_main_obsolete_head,
+        let mut lines = vec![];
+
+        if let Some(AncestorInfo { oid: _, distance }) = current_node.ancestor_info {
+            lines.push(
+                StyledStringBuilder::new()
+                    .append_plain(glyphs.commit_omitted)
+                    .append_plain(" ")
+                    .append_styled(
+                        &Pluralize {
+                            determiner: None,
+                            amount: distance,
+                            unit: ("omitted commit", "omitted commits"),
+                        }
+                        .to_string(),
+                        Effect::Dim,
+                    )
+                    .build(),
+            );
+            lines.push(StyledString::plain(glyphs.vertical_ellipsis));
         };
 
-        let first_line = {
-            let mut first_line = StyledString::new();
-            first_line.append_plain(cursor);
-            first_line.append_plain(" ");
-            first_line.append(text);
+        if let [_, merge_parents @ ..] = current_node.parents.as_slice() {
+            if !merge_parents.is_empty() {
+                for merge_parent_oid in merge_parents {
+                    let merge_parent_node = &graph.nodes[merge_parent_oid];
+                    lines.push(
+                        StyledStringBuilder::new()
+                            .append_plain(last_child_line_char.unwrap_or(glyphs.line))
+                            .append_plain(" ")
+                            .append_styled(
+                                format!("{} (merge) ", glyphs.commit_merge),
+                                BaseColor::Blue.dark(),
+                            )
+                            .append(render_node_descriptors(
+                                glyphs,
+                                &merge_parent_node.object,
+                                commit_descriptors,
+                            )?)
+                            .build(),
+                    );
+                }
+                lines.push(StyledString::plain(format!(
+                    "{}{}",
+                    glyphs.line_with_offshoot, glyphs.merge,
+                )));
+            }
+        }
+
+        lines.push({
+            let cursor = match (current_node.is_main, current_node.is_obsolete, is_head) {
+                (false, false, false) => glyphs.commit_visible,
+                (false, false, true) => glyphs.commit_visible_head,
+                (false, true, false) => glyphs.commit_obsolete,
+                (false, true, true) => glyphs.commit_obsolete_head,
+                (true, false, false) => glyphs.commit_main,
+                (true, false, true) => glyphs.commit_main_head,
+                (true, true, false) => glyphs.commit_main_obsolete,
+                (true, true, true) => glyphs.commit_main_obsolete_head,
+            };
+            let text = render_node_descriptors(glyphs, &current_node.object, commit_descriptors)?;
+            let first_line = StyledStringBuilder::new()
+                .append_plain(cursor)
+                .append_plain(" ")
+                .append(text)
+                .build();
             if is_head {
                 set_effect(first_line, Effect::Bold)
             } else {
                 first_line
             }
+        });
+
+        if current_node.num_omitted_descendants > 0 {
+            lines.push(StyledString::plain(glyphs.vertical_ellipsis));
+            lines.push(
+                StyledStringBuilder::new()
+                    .append_plain(glyphs.commit_omitted)
+                    .append_plain(" ")
+                    .append_styled(
+                        &Pluralize {
+                            determiner: None,
+                            amount: current_node.num_omitted_descendants,
+                            unit: ("omitted descendant commit", "omitted descendant commits"),
+                        }
+                        .to_string(),
+                        Effect::Dim,
+                    )
+                    .build(),
+            );
         };
 
-        let mut lines = vec![first_line];
-        let children: Vec<_> = current_node
+        let children: Vec<ChildInfo> = current_node
             .children
             .iter()
-            .filter(|child_oid| graph.nodes.contains_key(child_oid))
-            .copied()
+            .filter(
+                |ChildInfo {
+                     oid,
+                     is_merge_child: _,
+                 }| graph.nodes.contains_key(oid),
+            )
+            .cloned()
             .collect();
-        for (child_idx, child_oid) in children.iter().enumerate() {
+        let descendants: HashSet<ChildInfo> = current_node
+            .descendants
+            .iter()
+            .filter(
+                |ChildInfo {
+                     oid,
+                     is_merge_child: _,
+                 }| graph.nodes.contains_key(oid),
+            )
+            .cloned()
+            .collect();
+        for (child_idx, child_info) in children.iter().chain(descendants.iter()).enumerate() {
+            let ChildInfo {
+                oid: child_oid,
+                is_merge_child,
+            } = child_info;
             if root_oids.contains(child_oid) {
                 // Will be rendered by the parent.
                 continue;
             }
-
-            if child_idx == children.len() - 1 {
-                let line = match last_child_line_char {
-                    Some(_) => StyledString::plain(format!(
-                        "{}{}",
-                        glyphs.line_with_offshoot, glyphs.slash
-                    )),
-
-                    None => StyledString::plain(glyphs.line.to_string()),
-                };
-                lines.push(line)
-            } else {
-                lines.push(StyledString::plain(format!(
-                    "{}{}",
-                    glyphs.line_with_offshoot, glyphs.slash
-                )))
+            if *is_merge_child {
+                // lines.push(StyledString::plain(format!(
+                //     "{}{}",
+                //     glyphs.line_with_offshoot, glyphs.split
+                // )));
+                lines.push(
+                    StyledStringBuilder::new()
+                        // .append_plain(last_child_line_char.unwrap_or(glyphs.line))
+                        // .append_plain(" ")
+                        .append_styled(
+                            format!("{} (merge) ", glyphs.commit_merge),
+                            BaseColor::Blue.dark(),
+                        )
+                        .append(render_node_descriptors(
+                            glyphs,
+                            &graph.nodes[child_oid].object,
+                            commit_descriptors,
+                        )?)
+                        .build(),
+                );
+                continue;
             }
 
+            let is_last_child = child_idx == (children.len() + descendants.len()) - 1;
+            lines.push(StyledString::plain(
+                if !is_last_child || last_child_line_char.is_some() {
+                    format!("{}{}", glyphs.line_with_offshoot, glyphs.split)
+                } else if current_node.descendants.is_empty() {
+                    glyphs.line.to_string()
+                } else {
+                    glyphs.vertical_ellipsis.to_string()
+                },
+            ));
+
             let child_output = get_child_output(
                 glyphs,
                 graph,
@@ -403,7 +615,7 @@ mod render {
                 None,
             )?;
             for child_line in child_output {
-                let line = if child_idx == children.len() - 1 {
+                let line = if is_last_child {
                     match last_child_line_char {
                         Some(last_child_line_char) => StyledStringBuilder::new()
                             .append_plain(format!("{} ", last_child_line_char))
@@ -467,13 +679,10 @@ mod render {
             let last_child_line_char = {
                 if root_idx == root_oids.len() - 1 {
                     None
+                } else if has_real_parent(root_oids[root_idx + 1], *root_oid)? {
+                    Some(glyphs.line)
                 } else {
-                    let next_root_oid = root_oids[root_idx + 1];
-                    if has_real_parent(next_root_oid, *root_oid)? {
-                        Some(glyphs.line)
-                    } else {
-                        Some(glyphs.vertical_ellipsis)
-                    }
+                    Some(glyphs.vertical_ellipsis)
                 }
             };
 
@@ -502,7 +711,7 @@ mod render {
         head_oid: Option<NonZeroOid>,
         commit_descriptors: &mut [&mut dyn NodeDescriptor],
     ) -> eyre::Result<Vec<StyledString>> {
-        let root_oids = split_commit_graph_by_roots(effects, repo, dag, graph);
+        let root_oids = split_commit_graph_by_roots(repo, dag, graph);
         let lines = get_output(
             effects.get_glyphs(),
             dag,
@@ -517,26 +726,24 @@ mod render {
     /// Options for rendering the smartlog.
     #[derive(Debug)]
     pub struct SmartlogOptions {
-        /// Whether to also show commits in the smartlog which would normally not be
-        /// visible.
-        pub show_hidden_commits: bool,
-
         /// The point in time at which to show the smartlog. If not provided,
         /// renders the smartlog as of the current time. If negative, is treated
         /// as an offset from the current event.
         pub event_id: Option<isize>,
 
-        /// The commits to render. These commits and their ancestors up to the
-        /// main branch will be rendered.
+        /// The commits to render. These commits, plus any related commits, will
+        /// be rendered.
         pub revset: Revset,
+
+        pub resolve_revset_options: ResolveRevsetOptions,
     }
 
     impl Default for SmartlogOptions {
         fn default() -> Self {
             Self {
-                show_hidden_commits: Default::default(),
                 event_id: Default::default(),
-                revset: Revset("draft()".to_string()),
+                revset: Revset::default_smartlog_revset(),
+                resolve_revset_options: Default::default(),
             }
         }
     }
@@ -550,9 +757,9 @@ pub fn smartlog(
     options: &SmartlogOptions,
 ) -> eyre::Result<ExitCode> {
     let SmartlogOptions {
-        show_hidden_commits,
         event_id,
         revset,
+        resolve_revset_options,
     } = options;
 
     let repo = Repo::from_dir(&git_run_info.working_directory)?;
@@ -583,34 +790,24 @@ pub fn smartlog(
         &references_snapshot,
     )?;
 
-    let observed_commits = {
-        // For the purpose of resolving the revset expression, we may
-        // temporarily clear the DAG's obsolete commit set. However, when we
-        // render the smartlog later, we want to have the original obsolete
-        // commit set so that we can correctly identify which of the nodes are
-        // obsolete commits.
-        let mut old_obsolete_commits = CommitSet::empty();
-        if *show_hidden_commits {
-            swap(&mut dag.obsolete_commits, &mut old_obsolete_commits);
+    let commits = match resolve_commits(
+        effects,
+        &repo,
+        &mut dag,
+        &[revset.clone()],
+        resolve_revset_options,
+    ) {
+        Ok(result) => match result.as_slice() {
+            [commit_set] => commit_set.clone(),
+            other => panic!(
+                "Expected exactly 1 result from resolve commits, got: {:?}",
+                other
+            ),
+        },
+        Err(err) => {
+            err.describe(effects)?;
+            return Ok(ExitCode(1));
         }
-        let observed_commits = match resolve_commits(effects, &repo, &mut dag, vec![revset.clone()])
-        {
-            Ok(result) => match result.as_slice() {
-                [commit_set] => commit_set.clone(),
-                other => panic!(
-                    "Expected exactly 1 result from resolve commits, got: {:?}",
-                    other
-                ),
-            },
-            Err(err) => {
-                err.describe(effects)?;
-                return Ok(ExitCode(1));
-            }
-        };
-        if *show_hidden_commits {
-            swap(&mut dag.obsolete_commits, &mut old_obsolete_commits);
-        }
-        observed_commits
     };
 
     let graph = make_smartlog_graph(
@@ -619,8 +816,7 @@ pub fn smartlog(
         &dag,
         &event_replayer,
         event_cursor,
-        &observed_commits,
-        !show_hidden_commits,
+        &commits,
     )?;
 
     let lines = render_graph(
@@ -650,11 +846,13 @@ pub fn smartlog(
         writeln!(
             effects.get_output_stream(),
             "{}",
-            printable_styled_string(effects.get_glyphs(), line)?
+            effects.get_glyphs().render(line)?
         )?;
     }
 
-    if !show_hidden_commits && get_hint_enabled(&repo, Hint::SmartlogFixAbandoned)? {
+    if !resolve_revset_options.show_hidden_commits
+        && get_hint_enabled(&repo, Hint::SmartlogFixAbandoned)?
+    {
         let commits_with_abandoned_children: CommitSet = graph
             .nodes
             .iter()
@@ -669,12 +867,12 @@ pub fn smartlog(
             })
             .collect();
         let children = dag.query().children(commits_with_abandoned_children)?;
-        let num_abandoned_children = children.difference(&dag.obsolete_commits).count()?;
+        let num_abandoned_children = children.difference(&dag.query_obsolete_commits()).count()?;
         if num_abandoned_children > 0 {
             writeln!(
                 effects.get_output_stream(),
                 "{}: there {} in your commit graph",
-                style("hint").blue().bold(),
+                effects.get_glyphs().render(get_hint_string())?,
                 Pluralize {
                     determiner: Some(("is", "are")),
                     amount: num_abandoned_children,
@@ -684,7 +882,7 @@ pub fn smartlog(
             writeln!(
                 effects.get_output_stream(),
                 "{}: to fix this, run: git restack",
-                style("hint").blue().bold(),
+                effects.get_glyphs().render(get_hint_string())?,
             )?;
             print_hint_suppression_notice(effects, Hint::SmartlogFixAbandoned)?;
         }
