@@ -1,8 +1,7 @@
 //! The command-line options for `git-branchless`.
 
-use clap::{ArgEnum, Args, Command as ClapCommand, IntoApp, Parser};
+use clap::{Args, Command as ClapCommand, CommandFactory, Parser, ValueEnum};
 use lib::git::NonZeroOid;
-use man::Arg;
 use std::fmt::Display;
 use std::path::{Path, PathBuf};
 use std::str::FromStr;
@@ -12,6 +11,13 @@ use std::str::FromStr;
 #[derive(Clone, Debug)]
 pub struct Revset(pub String);
 
+impl Revset {
+    /// The default revset to render in the smartlog if no revset is provided by the user.
+    pub fn default_smartlog_revset() -> Self {
+        Self("((draft() | branches() | @) % main()) | branches() | @".to_string())
+    }
+}
+
 impl FromStr for Revset {
     type Err = std::convert::Infallible;
 
@@ -35,6 +41,14 @@ pub enum WrappedCommand {
     WrappedCommand(Vec<String>),
 }
 
+/// Options for resolving revset expressions.
+#[derive(Args, Debug, Default)]
+pub struct ResolveRevsetOptions {
+    /// Include hidden commits in the results of evaluating revset expressions.
+    #[clap(action, long = "hidden")]
+    pub show_hidden_commits: bool,
+}
+
 /// Options for moving commits.
 #[derive(Args, Debug)]
 pub struct MoveOptions {
@@ -45,7 +59,7 @@ pub struct MoveOptions {
 
     /// Only attempt to perform an in-memory rebase. If it fails, do not
     /// attempt an on-disk rebase.
-    #[clap(action, long = "in-memory", conflicts_with_all(&["force-on-disk", "merge"]))]
+    #[clap(action, long = "in-memory", conflicts_with_all(&["force_on_disk", "merge"]))]
     pub force_in_memory: bool,
 
     /// Skip attempting to use an in-memory rebase, and try an
@@ -124,14 +138,14 @@ pub struct TraverseCommitsOptions {
 
 /// Options for checking out a commit.
 #[derive(Args, Debug)]
-pub struct CheckoutOptions {
+pub struct SwitchOptions {
     /// Interactively select a commit to check out.
     #[clap(action, short = 'i', long = "interactive")]
     pub interactive: bool,
 
     /// When checking out the target commit, also create a branch with the
     /// provided name pointing to that commit.
-    #[clap(value_parser, short = 'b', long = "branch")]
+    #[clap(value_parser, short = 'c', long = "create")]
     pub branch_name: Option<String>,
 
     /// Forcibly switch commits, discarding any working copy changes if
@@ -169,13 +183,6 @@ pub enum Command {
     /// report.
     BugReport,
 
-    /// Check out a given commit.
-    Checkout {
-        /// Options for checking out a commit.
-        #[clap(flatten)]
-        checkout_options: CheckoutOptions,
-    },
-
     /// Run internal garbage collection.
     Gc,
 
@@ -185,6 +192,10 @@ pub enum Command {
         #[clap(value_parser)]
         revsets: Vec<Revset>,
 
+        /// Options for resolving revset expressions.
+        #[clap(flatten)]
+        resolve_revset_options: ResolveRevsetOptions,
+
         /// Also delete any branches that are abandoned as a result of this hide.
         #[clap(action, short = 'D', long = "delete-branches")]
         delete_branches: bool,
@@ -319,14 +330,18 @@ pub enum Command {
         #[clap(value_parser, short = 'd', long = "dest")]
         dest: Option<Revset>,
 
-        /// Insert the subtree between the destination and it's children, if any.
-        /// Only supported if the moved subtree has a single head.
-        #[clap(action, short = 'I', long = "insert")]
-        insert: bool,
+        /// Options for resolving revset expressions.
+        #[clap(flatten)]
+        resolve_revset_options: ResolveRevsetOptions,
 
         /// Options for moving commits.
         #[clap(flatten)]
         move_options: MoveOptions,
+
+        /// Insert the subtree between the destination and it's children, if any.
+        /// Only supported if the moved subtree has a single head.
+        #[clap(action, short = 'I', long = "insert")]
+        insert: bool,
     },
 
     /// Move to a later commit in the current stack.
@@ -356,13 +371,17 @@ pub enum Command {
         #[clap(value_parser)]
         revset: Revset,
 
+        /// Options for resolving revset expressions.
+        #[clap(flatten)]
+        resolve_revset_options: ResolveRevsetOptions,
+
         /// Print the branches attached to the resulting commits, rather than the commits themselves.
         #[clap(action, short = 'b', long = "branches")]
         show_branches: bool,
 
         /// Print the OID of each matching commit, one per line. This output is
         /// stable for use in scripts.
-        #[clap(action, short = 'r', long = "raw", conflicts_with("show-branches"))]
+        #[clap(action, short = 'r', long = "raw", conflicts_with("show_branches"))]
         raw: bool,
     },
 
@@ -379,7 +398,11 @@ pub enum Command {
         /// The IDs of the abandoned commits whose descendants should be
         /// restacked. If not provided, all abandoned commits are restacked.
         #[clap(value_parser)]
-        commits: Vec<Revset>,
+        revsets: Vec<Revset>,
+
+        /// Options for resolving revset expressions.
+        #[clap(flatten)]
+        resolve_revset_options: ResolveRevsetOptions,
 
         /// Options for moving commits.
         #[clap(flatten)]
@@ -409,6 +432,10 @@ pub enum Command {
         #[clap(value_parser)]
         revsets: Vec<Revset>,
 
+        /// Options for resolving revset expressions.
+        #[clap(flatten)]
+        resolve_revset_options: ResolveRevsetOptions,
+
         /// Force rewording public commits, even though other people may have access to
         /// those commits.
         #[clap(action, short = 'f', long = "force-rewrite", visible_alias = "fr")]
@@ -434,20 +461,20 @@ pub enum Command {
 
     /// Display a nice graph of the commits you've recently worked on.
     Smartlog {
-        /// Also show commits which have been hidden.
-        #[clap(action, long = "hidden")]
-        show_hidden_commits: bool,
-
         /// The point in time at which to show the smartlog. If not provided,
         /// renders the smartlog as of the current time. If negative, is treated
         /// as an offset from the current event.
         #[clap(value_parser, long = "event-id")]
         event_id: Option<isize>,
 
-        /// The commits to render. These commits and their ancestors up to the
-        /// main branch will be rendered.
-        #[clap(value_parser, default_value = "draft()")]
-        revset: Revset,
+        /// The commits to render. These commits, plus any related commits, will
+        /// be rendered.
+        #[clap(value_parser)]
+        revset: Option<Revset>,
+
+        /// Options for resolving revset expressions.
+        #[clap(flatten)]
+        resolve_revset_options: ResolveRevsetOptions,
     },
 
     #[clap(hide = true)]
@@ -473,6 +500,17 @@ pub enum Command {
         /// pushed.
         #[clap(value_parser, default_value = "stack()")]
         revset: Revset,
+
+        /// Options for resolving revset expressions.
+        #[clap(flatten)]
+        resolve_revset_options: ResolveRevsetOptions,
+    },
+
+    /// Switch to the provided branch or commit.
+    Switch {
+        /// Options for switching.
+        #[clap(flatten)]
+        switch_options: SwitchOptions,
     },
 
     /// Move any local commit stacks on top of the main branch.
@@ -486,7 +524,7 @@ pub enum Command {
             visible_short_alias = 'u',
             visible_alias = "--update"
         )]
-        update_refs: bool,
+        pull: bool,
 
         /// Options for moving commits.
         #[clap(flatten)]
@@ -496,6 +534,17 @@ pub enum Command {
         /// no commits are provided, all draft commits will be synced.
         #[clap(value_parser)]
         revsets: Vec<Revset>,
+
+        /// Options for resolving revset expressions.
+        #[clap(flatten)]
+        resolve_revset_options: ResolveRevsetOptions,
+    },
+
+    /// Run a command on each commit in a given set and aggregate the results.
+    Test {
+        /// The subcommand to run.
+        #[clap(subcommand)]
+        subcommand: TestSubcommand,
     },
 
     /// Browse or return to a previous state of the repository.
@@ -516,6 +565,10 @@ pub enum Command {
         #[clap(value_parser)]
         revsets: Vec<Revset>,
 
+        /// Options for resolving revset expressions.
+        #[clap(flatten)]
+        resolve_revset_options: ResolveRevsetOptions,
+
         /// Also recursively unhide all children commits of the provided commits.
         #[clap(action, short = 'r', long = "recursive")]
         recursive: bool,
@@ -534,7 +587,7 @@ pub enum Command {
 }
 
 /// Whether to display terminal colors.
-#[derive(ArgEnum, Clone)]
+#[derive(ValueEnum, Clone)]
 pub enum ColorSetting {
     /// Automatically determine whether to display colors from the terminal and environment variables.
     /// This is the default behavior.
@@ -545,6 +598,20 @@ pub enum ColorSetting {
     Never,
 }
 
+/// How to execute tests.
+#[derive(Clone, Copy, Debug, ValueEnum)]
+pub enum TestExecutionStrategy {
+    /// Default. Run the tests in the working copy. This requires a clean working copy. This is
+    /// useful if you want to reuse build artifacts in the current directory.
+    WorkingCopy,
+
+    /// Run the tests in a separate worktree (managed by git-branchless). This is useful if you want
+    /// to run tests in parallel, or if you want to run tests on a different commit without
+    /// invalidating build artifacts in the current directory, or if you want to run tests while
+    /// your working copy is dirty.
+    Worktree,
+}
+
 /// Branchless workflow for Git.
 ///
 /// See the documentation at <https://github.com/arxanas/git-branchless/wiki>.
@@ -557,7 +624,7 @@ pub struct Opts {
     pub working_directory: Option<PathBuf>,
 
     /// Flag to force enable or disable terminal colors.
-    #[clap(value_parser, long = "color", arg_enum, global = true)]
+    #[clap(value_parser, long = "color", value_enum, global = true)]
     pub color: Option<ColorSetting>,
 
     /// The `git-branchless` subcommand to run.
@@ -582,6 +649,77 @@ pub enum SnapshotSubcommand {
     },
 }
 
+/// `test` subcommands.
+#[derive(Parser)]
+pub enum TestSubcommand {
+    /// Run a given command on a set of commits and present the successes and failures.
+    Run {
+        /// An ad-hoc command to execute on each commit.
+        #[clap(value_parser, short = 'x', long = "exec")]
+        exec: Option<String>,
+
+        /// The test command alias for the command to execute on each commit. Set with
+        /// `git config branchless.test.alias.<name> <command>`.
+        #[clap(value_parser, short = 'c', long = "command", conflicts_with("exec"))]
+        command: Option<String>,
+
+        /// The set of commits to test.
+        #[clap(value_parser, default_value = "stack()")]
+        revset: Revset,
+
+        /// Options for resolving revset expressions.
+        #[clap(flatten)]
+        resolve_revset_options: ResolveRevsetOptions,
+
+        /// Show the test output as well.
+        #[clap(short = 'v', long = "verbose", action = clap::ArgAction::Count)]
+        verbosity: u8,
+
+        /// How to execute the tests.
+        #[clap(short = 's', long = "strategy")]
+        strategy: Option<TestExecutionStrategy>,
+
+        /// How many jobs to execute in parallel. The value `0` indicates to use all CPUs.
+        #[clap(short = 'j', long = "jobs")]
+        jobs: Option<usize>,
+    },
+
+    /// Show the results of a set of previous test runs.
+    Show {
+        /// An ad-hoc command to execute on each commit.
+        #[clap(value_parser, short = 'x', long = "exec")]
+        exec: Option<String>,
+
+        /// The test command alias for the command to execute on each commit. Set with
+        /// `git config branchless.test.alias.<name> <command>`.
+        #[clap(value_parser, short = 'c', long = "command", conflicts_with("exec"))]
+        command: Option<String>,
+
+        /// The set of commits to show the test output for.
+        #[clap(value_parser, default_value = "stack()")]
+        revset: Revset,
+
+        /// Options for resolving revset expressions.
+        #[clap(flatten)]
+        resolve_revset_options: ResolveRevsetOptions,
+
+        /// Show the test output as well.
+        #[clap(short = 'v', long = "verbose", action = clap::ArgAction::Count)]
+        verbosity: u8,
+    },
+
+    /// Clean any cached test results.
+    Clean {
+        /// The set of commits whose results should be cleaned.
+        #[clap(value_parser, default_value = "stack()")]
+        revset: Revset,
+
+        /// Options for resolving revset expressions.
+        #[clap(flatten)]
+        resolve_revset_options: ResolveRevsetOptions,
+    },
+}
+
 /// Generate and write man-pages into the specified directory.
 ///
 /// The generated files are named things like `man1/git-branchless-smartlog.1`,
@@ -601,56 +739,12 @@ pub fn write_man_pages(man_dir: &Path) -> std::io::Result<()> {
 }
 
 fn generate_man_page(man1_dir: &Path, name: &str, command: &ClapCommand) -> std::io::Result<()> {
-    let mut manual = man::Manual::new(name);
-    if let Some(about) = command.get_about() {
-        manual = manual.about(about);
-    }
-
-    let authors = env!("CARGO_PKG_AUTHORS").split(':').map(|author_string| {
-        let (name, email) = match author_string.split_once(" <") {
-            Some(value) => value,
-            None => panic!(
-                "Invalid author specifier (should be Full Name <email@example.com>): {:?}",
-                author_string
-            ),
-        };
-
-        let email = email.strip_prefix('<').unwrap_or(email);
-        let email = email.strip_suffix('>').unwrap_or(email);
-        man::Author::new(name).email(email)
-    });
-    for author in authors {
-        manual = manual.author(author);
-    }
-
-    if let Some(long_about) = command.get_long_about() {
-        manual = manual.description(long_about);
-    }
-
-    for arg in command.get_positionals() {
-        manual = manual.arg(Arg::new(&format!("[{}]", arg.get_id().to_uppercase())));
-    }
-
-    for flag in command.get_opts() {
-        let opt = man::Opt::new(flag.get_id());
-        let opt = match flag.get_short() {
-            Some(short) => opt.short(&String::from(short)),
-            None => opt,
-        };
-        let opt = match flag.get_long() {
-            Some(long) => opt.long(long),
-            None => opt,
-        };
-        let opt = match flag.get_help() {
-            Some(help) => opt.help(help),
-            None => opt,
-        };
-        manual = manual.option(opt);
-    }
-
-    // FIXME: implement rest of man-page rendering.
-
+    let rendered_man_page = {
+        let mut buffer = Vec::new();
+        clap_mangen::Man::new(command.clone()).render(&mut buffer)?;
+        buffer
+    };
     let output_path = man1_dir.join(format!("{}.1", name));
-    std::fs::write(output_path, manual.render())?;
+    std::fs::write(output_path, rendered_man_page)?;
     Ok(())
 }
