@@ -1,11 +1,13 @@
 use bstr::ByteSlice;
 use eden_dag::DagAlgorithm;
 use lib::core::dag::CommitSet;
-use lib::git::{Commit, Repo};
+use lib::core::eventlog::{EventLogDb, EventReplayer};
+use lib::core::rewrite::find_rewrite_target;
+use lib::git::{Commit, MaybeZeroOid, NonZeroOid, Repo};
 use std::borrow::Cow;
 use std::collections::HashMap;
 use std::convert::TryFrom;
-use tracing::warn;
+use tracing::{instrument, warn};
 
 use eyre::Context as EyreContext;
 use lazy_static::lazy_static;
@@ -39,6 +41,8 @@ lazy_static! {
             ("roots", &fn_roots),
             ("heads", &fn_heads),
             ("branches", &fn_branches),
+            ("main", &fn_main),
+            ("public", &fn_public),
             ("draft", &fn_draft),
             ("stack", &fn_stack),
             ("message", &fn_message),
@@ -50,87 +54,117 @@ lazy_static! {
             ("committer.email", &fn_committer_email),
             ("committer.date", &fn_committer_date),
             ("exactly", &fn_exactly),
+            ("current", &fn_current),
         ];
         functions.iter().cloned().collect()
     };
 }
 
+#[instrument]
 fn fn_all(ctx: &mut Context, name: &str, args: &[Expr]) -> EvalResult {
     eval0(ctx, name, args)?;
-    Ok(ctx.query_active_commits()?.clone())
+    let visible_heads = ctx
+        .dag
+        .query_visible_heads()
+        .map_err(EvalError::OtherError)?;
+    let visible_commits = ctx.dag.query().ancestors(visible_heads.clone())?;
+    let visible_commits = ctx
+        .dag
+        .filter_visible_commits(visible_commits)
+        .map_err(EvalError::OtherError)?;
+    Ok(visible_commits)
 }
 
+#[instrument]
 fn fn_none(ctx: &mut Context, name: &str, args: &[Expr]) -> EvalResult {
     eval0(ctx, name, args)?;
     Ok(CommitSet::empty())
 }
 
+#[instrument]
 fn fn_union(ctx: &mut Context, name: &str, args: &[Expr]) -> EvalResult {
     let (lhs, rhs) = eval2(ctx, name, args)?;
     Ok(lhs.union(&rhs))
 }
 
+#[instrument]
 fn fn_intersection(ctx: &mut Context, name: &str, args: &[Expr]) -> EvalResult {
     let (lhs, rhs) = eval2(ctx, name, args)?;
     Ok(lhs.intersection(&rhs))
 }
 
+#[instrument]
 fn fn_difference(ctx: &mut Context, name: &str, args: &[Expr]) -> EvalResult {
     let (lhs, rhs) = eval2(ctx, name, args)?;
     Ok(lhs.difference(&rhs))
 }
 
+#[instrument]
 fn fn_only(ctx: &mut Context, name: &str, args: &[Expr]) -> EvalResult {
     let (lhs, rhs) = eval2(ctx, name, args)?;
     Ok(ctx.dag.query().only(lhs, rhs)?)
 }
 
+#[instrument]
 fn fn_range(ctx: &mut Context, name: &str, args: &[Expr]) -> EvalResult {
     let (lhs, rhs) = eval2(ctx, name, args)?;
     Ok(ctx.dag.query().range(lhs, rhs)?)
 }
 
+#[instrument]
 fn fn_not(ctx: &mut Context, name: &str, args: &[Expr]) -> EvalResult {
     let expr = eval1(ctx, name, args)?;
-    let active_commits = ctx.query_active_commits()?;
-    Ok(active_commits.difference(&expr))
+    let visible_heads = ctx
+        .dag
+        .query_visible_heads()
+        .map_err(EvalError::OtherError)?;
+    let visible_commits = ctx.dag.query().ancestors(visible_heads.clone())?;
+    Ok(visible_commits.difference(&expr))
 }
 
+#[instrument]
 fn fn_ancestors(ctx: &mut Context, name: &str, args: &[Expr]) -> EvalResult {
     let expr = eval1(ctx, name, args)?;
     Ok(ctx.dag.query().ancestors(expr)?)
 }
 
+#[instrument]
 fn fn_descendants(ctx: &mut Context, name: &str, args: &[Expr]) -> EvalResult {
     let expr = eval1(ctx, name, args)?;
     Ok(ctx.dag.query().descendants(expr)?)
 }
 
+#[instrument]
 fn fn_parents(ctx: &mut Context, name: &str, args: &[Expr]) -> EvalResult {
     let expr = eval1(ctx, name, args)?;
     Ok(ctx.dag.query().parents(expr)?)
 }
 
+#[instrument]
 fn fn_children(ctx: &mut Context, name: &str, args: &[Expr]) -> EvalResult {
     let expr = eval1(ctx, name, args)?;
     Ok(ctx.dag.query().children(expr)?)
 }
 
+#[instrument]
 fn fn_roots(ctx: &mut Context, name: &str, args: &[Expr]) -> EvalResult {
     let expr = eval1(ctx, name, args)?;
     Ok(ctx.dag.query().roots(expr)?)
 }
 
+#[instrument]
 fn fn_heads(ctx: &mut Context, name: &str, args: &[Expr]) -> EvalResult {
     let expr = eval1(ctx, name, args)?;
     Ok(ctx.dag.query().heads(expr)?)
 }
 
+#[instrument]
 fn fn_branches(ctx: &mut Context, name: &str, args: &[Expr]) -> EvalResult {
     eval0(ctx, name, args)?;
     Ok(ctx.dag.branch_commits.clone())
 }
 
+#[instrument]
 fn fn_parents_nth(ctx: &mut Context, name: &str, args: &[Expr]) -> EvalResult {
     let (lhs, n) = eval_number_rhs(ctx, name, args)?;
     let mut result = Vec::new();
@@ -152,6 +186,7 @@ fn fn_parents_nth(ctx: &mut Context, name: &str, args: &[Expr]) -> EvalResult {
     Ok(CommitSet::from_iter(result.into_iter()))
 }
 
+#[instrument]
 fn fn_nthancestor(ctx: &mut Context, name: &str, args: &[Expr]) -> EvalResult {
     let (lhs, n) = eval_number_rhs(ctx, name, args)?;
     let n: u64 = u64::try_from(n).unwrap();
@@ -172,15 +207,39 @@ fn fn_nthancestor(ctx: &mut Context, name: &str, args: &[Expr]) -> EvalResult {
     Ok(CommitSet::from_iter(result.into_iter()))
 }
 
+#[instrument]
+fn fn_main(ctx: &mut Context, name: &str, args: &[Expr]) -> EvalResult {
+    eval0(ctx, name, args)?;
+    Ok(ctx.dag.main_branch_commit.clone())
+}
+
+#[instrument]
+fn fn_public(ctx: &mut Context, name: &str, args: &[Expr]) -> EvalResult {
+    eval0(ctx, name, args)?;
+    let public_commits = ctx
+        .dag
+        .query_public_commits_slow()
+        .map_err(EvalError::OtherError)?;
+    Ok(public_commits.clone())
+}
+
+#[instrument]
 fn fn_draft(ctx: &mut Context, name: &str, args: &[Expr]) -> EvalResult {
     eval0(ctx, name, args)?;
-    let draft_commits = ctx.query_draft_commits()?;
+    let draft_commits = ctx
+        .dag
+        .query_draft_commits()
+        .map_err(EvalError::OtherError)?;
     Ok(draft_commits.clone())
 }
 
+#[instrument]
 fn fn_stack(ctx: &mut Context, name: &str, args: &[Expr]) -> EvalResult {
     let arg = eval0_or_1(ctx, name, args)?.unwrap_or_else(|| ctx.dag.head_commit.clone());
-    let draft_commits = ctx.query_draft_commits()?;
+    let draft_commits = ctx
+        .dag
+        .query_draft_commits()
+        .map_err(EvalError::OtherError)?;
     let stack_roots = ctx.dag.query().roots(draft_commits.clone())?;
     let stack_ancestors = ctx.dag.query().range(stack_roots, arg)?;
     let stack = ctx
@@ -204,6 +263,7 @@ fn fn_stack(ctx: &mut Context, name: &str, args: &[Expr]) -> EvalResult {
 
 type MatcherFn = dyn Fn(&Repo, &Commit) -> Result<bool, PatternError> + Sync + Send;
 
+#[instrument(skip(f))]
 fn make_pattern_matcher(
     ctx: &mut Context,
     name: &str,
@@ -233,6 +293,7 @@ fn make_pattern_matcher(
     Ok(matcher)
 }
 
+#[instrument]
 fn fn_message(ctx: &mut Context, name: &str, args: &[Expr]) -> EvalResult {
     let pattern = eval1_pattern(ctx, name, args)?;
     make_pattern_matcher(
@@ -258,6 +319,7 @@ fn fn_message(ctx: &mut Context, name: &str, args: &[Expr]) -> EvalResult {
     )
 }
 
+#[instrument]
 fn fn_path_changed(ctx: &mut Context, name: &str, args: &[Expr]) -> EvalResult {
     let pattern = eval1_pattern(ctx, name, args)?;
     make_pattern_matcher(
@@ -295,6 +357,7 @@ fn fn_path_changed(ctx: &mut Context, name: &str, args: &[Expr]) -> EvalResult {
     )
 }
 
+#[instrument]
 fn fn_author_name(ctx: &mut Context, name: &str, args: &[Expr]) -> EvalResult {
     let pattern = eval1_pattern(ctx, name, args)?;
     make_pattern_matcher(
@@ -310,6 +373,7 @@ fn fn_author_name(ctx: &mut Context, name: &str, args: &[Expr]) -> EvalResult {
     )
 }
 
+#[instrument]
 fn fn_author_email(ctx: &mut Context, name: &str, args: &[Expr]) -> EvalResult {
     let pattern = eval1_pattern(ctx, name, args)?;
     make_pattern_matcher(
@@ -325,6 +389,7 @@ fn fn_author_email(ctx: &mut Context, name: &str, args: &[Expr]) -> EvalResult {
     )
 }
 
+#[instrument]
 fn fn_author_date(ctx: &mut Context, name: &str, args: &[Expr]) -> EvalResult {
     let pattern = eval1_pattern(ctx, name, args)?;
     make_pattern_matcher(
@@ -338,6 +403,7 @@ fn fn_author_date(ctx: &mut Context, name: &str, args: &[Expr]) -> EvalResult {
     )
 }
 
+#[instrument]
 fn fn_committer_name(ctx: &mut Context, name: &str, args: &[Expr]) -> EvalResult {
     let pattern = eval1_pattern(ctx, name, args)?;
     make_pattern_matcher(
@@ -353,6 +419,7 @@ fn fn_committer_name(ctx: &mut Context, name: &str, args: &[Expr]) -> EvalResult
     )
 }
 
+#[instrument]
 fn fn_committer_email(ctx: &mut Context, name: &str, args: &[Expr]) -> EvalResult {
     let pattern = eval1_pattern(ctx, name, args)?;
     make_pattern_matcher(
@@ -368,6 +435,7 @@ fn fn_committer_email(ctx: &mut Context, name: &str, args: &[Expr]) -> EvalResul
     )
 }
 
+#[instrument]
 fn fn_committer_date(ctx: &mut Context, name: &str, args: &[Expr]) -> EvalResult {
     let pattern = eval1_pattern(ctx, name, args)?;
     make_pattern_matcher(
@@ -381,6 +449,7 @@ fn fn_committer_date(ctx: &mut Context, name: &str, args: &[Expr]) -> EvalResult
     )
 }
 
+#[instrument]
 fn fn_exactly(ctx: &mut Context, name: &str, args: &[Expr]) -> EvalResult {
     let (lhs, expected_len) = eval_number_rhs(ctx, name, args)?;
     let actual_len: usize = lhs
@@ -398,3 +467,56 @@ fn fn_exactly(ctx: &mut Context, name: &str, args: &[Expr]) -> EvalResult {
         })
     }
 }
+
+#[instrument]
+fn fn_current(ctx: &mut Context, name: &str, args: &[Expr]) -> EvalResult {
+    let mut dag = ctx
+        .dag
+        .clear_obsolete_commits(ctx.repo)
+        .map_err(EvalError::OtherError)?;
+    let mut ctx = Context {
+        effects: ctx.effects,
+        repo: ctx.repo,
+        dag: &mut dag,
+    };
+    let expr = eval1(&mut ctx, name, args)?;
+
+    let conn = ctx.repo.get_db_conn()?;
+    let event_log_db = EventLogDb::new(&conn)
+        .wrap_err("Connecting to event log")
+        .map_err(EvalError::OtherError)?;
+    let event_replayer = EventReplayer::from_event_log_db(ctx.effects, ctx.repo, &event_log_db)
+        .wrap_err("Retrieving event replayer")
+        .map_err(EvalError::OtherError)?;
+    let event_cursor = event_replayer.make_default_cursor();
+
+    let mut result = Vec::new();
+    for vertex in expr
+        .iter()
+        .wrap_err("Iterating commit set")
+        .map_err(EvalError::OtherError)?
+    {
+        let vertex = vertex
+            .wrap_err("Evaluating vertex")
+            .map_err(EvalError::OtherError)?;
+
+        let oid = NonZeroOid::try_from(vertex)
+            .wrap_err("Converting vertex to oid")
+            .map_err(EvalError::OtherError)?;
+
+        match find_rewrite_target(&event_replayer, event_cursor, oid) {
+            Some(new_commit_oid) => match new_commit_oid {
+                MaybeZeroOid::NonZero(new_commit_oid) => {
+                    // commit rewritten as new_commit_oid
+                    result.push(new_commit_oid);
+                }
+                MaybeZeroOid::Zero => {
+                    // commit deleted, skip
+                }
+            },
+            // commit not rewritten
+            None => result.push(oid),
+        }
+    }
+    Ok(result.into_iter().collect::<CommitSet>())
+}
