@@ -4,10 +4,8 @@ use std::num::ParseIntError;
 use std::sync::Arc;
 
 use eden_dag::errors::BackendError;
-use eden_dag::DagAlgorithm;
 use itertools::Itertools;
 use lib::core::effects::{Effects, OperationType};
-use once_cell::unsync::OnceCell;
 use thiserror::Error;
 
 use lib::core::dag::{CommitSet, Dag};
@@ -25,62 +23,6 @@ pub(super) struct Context<'a> {
     pub effects: &'a Effects,
     pub repo: &'a Repo,
     pub dag: &'a mut Dag,
-    pub public_commits: OnceCell<CommitSet>,
-    pub active_heads: OnceCell<CommitSet>,
-    pub active_commits: OnceCell<CommitSet>,
-    pub draft_commits: OnceCell<CommitSet>,
-}
-
-impl Context<'_> {
-    #[instrument]
-    pub fn query_public_commits(&self) -> Result<&CommitSet, EvalError> {
-        self.public_commits.get_or_try_init(|| {
-            let public_commits = self
-                .dag
-                .query_public_commits()
-                .map_err(EvalError::OtherError)?;
-            Ok(public_commits)
-        })
-    }
-
-    #[instrument]
-    pub fn query_active_heads(&self) -> Result<&CommitSet, EvalError> {
-        self.active_heads.get_or_try_init(|| {
-            let public_commits = self.query_public_commits()?;
-            let active_heads = self
-                .dag
-                .query_active_heads(
-                    public_commits,
-                    &self
-                        .dag
-                        .observed_commits
-                        .difference(&self.dag.obsolete_commits),
-                )
-                .map_err(EvalError::OtherError)?;
-            Ok(active_heads)
-        })
-    }
-
-    #[instrument]
-    pub fn query_active_commits(&self) -> Result<&CommitSet, EvalError> {
-        self.active_commits.get_or_try_init(|| {
-            let active_heads = self.query_active_heads()?;
-            let active_commits = self.dag.query().ancestors(active_heads.clone())?;
-            Ok(active_commits)
-        })
-    }
-
-    #[instrument]
-    pub fn query_draft_commits(&self) -> Result<&CommitSet, EvalError> {
-        self.draft_commits.get_or_try_init(|| {
-            let public_commits = self.query_public_commits()?;
-            let active_heads = self.query_active_heads()?;
-            Ok(self
-                .dag
-                .query()
-                .only(active_heads.clone(), public_commits.clone())?)
-        })
-    }
 }
 
 #[derive(Debug, Error)]
@@ -170,10 +112,6 @@ pub fn eval(effects: &Effects, repo: &Repo, dag: &mut Dag, expr: &Expr) -> EvalR
         effects: &effects,
         repo,
         dag,
-        public_commits: Default::default(),
-        active_heads: Default::default(),
-        active_commits: Default::default(),
-        draft_commits: Default::default(),
     };
     let commits = eval_inner(&mut ctx, expr)?;
     Ok(commits)
@@ -183,20 +121,31 @@ pub fn eval(effects: &Effects, repo: &Repo, dag: &mut Dag, expr: &Expr) -> EvalR
 fn eval_inner(ctx: &mut Context, expr: &Expr) -> EvalResult {
     match expr {
         Expr::Name(name) => eval_name(ctx, name),
-        Expr::FunctionCall(name, args) => eval_fn(ctx, name, args),
+        Expr::FunctionCall(name, args) => {
+            let result = eval_fn(ctx, name, args)?;
+            let result = ctx
+                .dag
+                .filter_visible_commits(result)
+                .map_err(EvalError::OtherError)?;
+            Ok(result)
+        }
     }
 }
 
+#[instrument]
 pub(super) fn eval_name(ctx: &mut Context, name: &str) -> EvalResult {
     if name == "." || name == "@" {
-        let head_info = ctx.repo.get_head_info();
-        if let Ok(ResolvedReferenceInfo {
-            oid: Some(oid),
-            reference_name: _,
-        }) = head_info
-        {
-            return Ok(oid.into());
-        }
+        let head_info = ctx.repo.get_head_info()?;
+        return match head_info {
+            ResolvedReferenceInfo {
+                oid: Some(oid),
+                reference_name: _,
+            } => Ok(oid.into()),
+            ResolvedReferenceInfo {
+                oid: None,
+                reference_name: _,
+            } => Ok(CommitSet::empty()),
+        };
     }
 
     let commit = ctx.repo.revparse_single_commit(name);
@@ -223,6 +172,7 @@ pub(super) fn eval_name(ctx: &mut Context, name: &str) -> EvalResult {
     Ok(commit_set)
 }
 
+#[instrument]
 pub(super) fn eval_fn(ctx: &mut Context, name: &str, args: &[Expr]) -> EvalResult {
     if let Some(function) = FUNCTIONS.get(name) {
         return function(ctx, name, args);
@@ -246,8 +196,8 @@ pub(super) fn eval_fn(ctx: &mut Context, name: &str, args: &[Expr]) -> EvalResul
             .map(|(i, arg)| (format!("${}", i + 1), arg.clone()))
             .collect();
         let alias_expr = alias_expr.replace_names(&arg_map);
-        let commits = eval_inner(ctx, &alias_expr);
-        return commits;
+        let commits = eval_inner(ctx, &alias_expr)?;
+        return Ok(commits);
     }
 
     Err(EvalError::UnboundFunction {
@@ -309,6 +259,7 @@ pub(super) fn eval1(ctx: &mut Context, function_name: &str, args: &[Expr]) -> Ev
     }
 }
 
+#[instrument]
 pub(super) fn eval1_pattern(
     _ctx: &mut Context,
     function_name: &str,
@@ -350,6 +301,7 @@ pub(super) fn eval2(
     }
 }
 
+#[instrument]
 pub(super) fn eval_number_rhs(
     ctx: &mut Context,
     function_name: &str,
@@ -560,6 +512,50 @@ mod tests {
             "###);
         }
 
+        {
+            let expr = Expr::FunctionCall(Cow::Borrowed("main"), vec![]);
+            insta::assert_debug_snapshot!(eval_and_sort(&effects, &repo, &mut dag, &expr), @r###"
+            Ok(
+                [
+                    Commit {
+                        inner: Commit {
+                            id: bf0d52a607f693201512a43b6b5a70b2a275e0ad,
+                            summary: "create test4.txt",
+                        },
+                    },
+                ],
+            )
+            "###);
+        }
+
+        {
+            let expr = Expr::FunctionCall(Cow::Borrowed("public"), vec![]);
+            insta::assert_debug_snapshot!(eval_and_sort(&effects, &repo, &mut dag, &expr), @r###"
+            Ok(
+                [
+                    Commit {
+                        inner: Commit {
+                            id: f777ecc9b0db5ed372b2615695191a8a17f79f24,
+                            summary: "create initial.txt",
+                        },
+                    },
+                    Commit {
+                        inner: Commit {
+                            id: 62fc20d2a290daea0d52bdc2ed2ad4be6491010e,
+                            summary: "create test1.txt",
+                        },
+                    },
+                    Commit {
+                        inner: Commit {
+                            id: bf0d52a607f693201512a43b6b5a70b2a275e0ad,
+                            summary: "create test4.txt",
+                        },
+                    },
+                ],
+            )
+            "###);
+        }
+
         {
             let expr = Expr::FunctionCall(
                 Cow::Borrowed("stack"),
@@ -849,7 +845,7 @@ mod tests {
         let git = make_git()?;
         git.init_repo()?;
 
-        git.write_file("test1", "test\n")?;
+        git.write_file_txt("test1", "test\n")?;
         git.run(&["add", "test1.txt"])?;
         git.run_with_options(
             &["commit", "-m", "test1"],
@@ -869,7 +865,7 @@ mod tests {
             },
         )?;
 
-        git.write_file("test2", "test\n")?;
+        git.write_file_txt("test2", "test\n")?;
         git.run(&["add", "test2.txt"])?;
         git.run_with_options(
             &["commit", "-m", "test2"],
@@ -1069,6 +1065,90 @@ mod tests {
         Ok(())
     }
 
+    #[test]
+    fn test_eval_current() -> eyre::Result<()> {
+        let git = make_git()?;
+        git.init_repo()?;
+
+        git.detach_head()?;
+        let test1_oid = git.commit_file("test1", 1)?;
+        let _test2_oid = git.commit_file("test2", 2)?;
+        let test3_oid = git.commit_file("test3", 3)?;
+        let test4_oid = git.commit_file("test4", 4)?;
+
+        git.run(&[
+            "move",
+            "-s",
+            &test3_oid.to_string(),
+            "-d",
+            &test1_oid.to_string(),
+        ])?;
+        git.run(&["reword", "-m", "test4 has been rewritten twice"])?;
+
+        let effects = Effects::new_suppress_for_test(Glyphs::text());
+        let repo = git.get_repo()?;
+        let conn = repo.get_db_conn()?;
+        let event_log_db = EventLogDb::new(&conn)?;
+        let event_replayer = EventReplayer::from_event_log_db(&effects, &repo, &event_log_db)?;
+        let event_cursor = event_replayer.make_default_cursor();
+        let references_snapshot = repo.get_references_snapshot()?;
+        let mut dag = Dag::open_and_sync(
+            &effects,
+            &repo,
+            &event_replayer,
+            event_cursor,
+            &references_snapshot,
+        )?;
+
+        {
+            let original_test3_oid = &test3_oid.to_string();
+            let original_test4_oid = &test4_oid.to_string();
+
+            let expr = Expr::FunctionCall(
+                Cow::Borrowed("union"),
+                vec![
+                    Expr::Name(Cow::Borrowed(original_test3_oid)),
+                    Expr::Name(Cow::Borrowed(original_test4_oid)),
+                ],
+            );
+            insta::assert_debug_snapshot!(eval_and_sort(&effects, &repo, &mut dag, &expr), @r###"
+            Ok(
+                [],
+            )
+            "###);
+
+            let expr = Expr::FunctionCall(
+                Cow::Borrowed("current"),
+                vec![Expr::FunctionCall(
+                    Cow::Borrowed("union"),
+                    vec![
+                        Expr::Name(Cow::Borrowed(original_test3_oid)),
+                        Expr::Name(Cow::Borrowed(original_test4_oid)),
+                    ],
+                )],
+            );
+            insta::assert_debug_snapshot!(eval_and_sort(&effects, &repo, &mut dag, &expr), @r###"
+            Ok(
+                [
+                    Commit {
+                        inner: Commit {
+                            id: 4838e49b08954becdd17c0900c1179c2c654c627,
+                            summary: "create test3.txt",
+                        },
+                    },
+                    Commit {
+                        inner: Commit {
+                            id: 619162078182d2c6d80ff604b81e7c2afc3295b7,
+                            summary: "test4 has been rewritten twice",
+                        },
+                    },
+                ],
+            )
+            "###);
+        }
+        Ok(())
+    }
+
     #[test]
     fn test_eval_aliases() -> eyre::Result<()> {
         let git = make_git()?;
