@@ -6,7 +6,9 @@ use crate::util::trim_lines;
 
 use eyre::Context;
 use lib::git::GitVersion;
-use lib::testing::{make_git, GitInitOptions, GitRunOptions};
+use lib::testing::{
+    make_git, make_git_worktree, GitInitOptions, GitRunOptions, GitWorktreeWrapper,
+};
 
 #[test]
 fn test_hook_installed() -> eyre::Result<()> {
@@ -182,7 +184,7 @@ fn test_old_git_version_warning() -> eyre::Result<()> {
         insta::assert_snapshot!(stdout, @r###"
         Created config file at <repo-path>/.git/branchless/config
         Auto-detected your main branch as: master
-        If this is incorrect, run: git config branchless.core.mainBranch <branch>
+        If this is incorrect, run: git branchless init --main-branch <branch>
         Installing hook: post-commit
         Installing hook: post-merge
         Installing hook: post-rewrite
@@ -227,7 +229,7 @@ fn test_init_basic() -> eyre::Result<()> {
         insta::assert_snapshot!(stdout, @r###"
         Created config file at <repo-path>/.git/branchless/config
         Auto-detected your main branch as: master
-        If this is incorrect, run: git config branchless.core.mainBranch <branch>
+        If this is incorrect, run: git branchless init --main-branch <branch>
         Installing hook: post-commit
         Installing hook: post-merge
         Installing hook: post-rewrite
@@ -325,7 +327,7 @@ fn test_main_branch_not_found_error_message() -> eyre::Result<()> {
 
       ━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━ SPANTRACE ━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━
 
-       0: git_branchless::commands::smartlog::smartlog with effects=<Output fancy=false> git_run_info=<GitRunInfo path_to_git="<git-executable>" working_directory="<repo-path>" env=not shown> options=SmartlogOptions { show_hidden_commits: false, event_id: None, revset: Revset("draft()") }
+       0: git_branchless::commands::smartlog::smartlog with effects=<Output fancy=false> git_run_info=<GitRunInfo path_to_git="<git-executable>" working_directory="<repo-path>" env=not shown> options=SmartlogOptions { event_id: None, revset: Revset("((draft() | branches() | @) % main()) | branches() | @"), resolve_revset_options: ResolveRevsetOptions { show_hidden_commits: false } }
           at some/file/path.rs:123
 
     Suggestion:
@@ -334,8 +336,10 @@ fn test_main_branch_not_found_error_message() -> eyre::Result<()> {
     These branches exist: []
     Either create it, or update the main branch setting by running:
 
-        git config branchless.core.mainBranch <branch>
+        git branchless init --main-branch <branch>
 
+    Note that remote main branches are no longer supported as of v0.6.0. See
+    https://github.com/arxanas/git-branchless/discussions/595 for more details.
 
     Backtrace omitted. Run with RUST_BACKTRACE=1 environment variable to display it.
     Run with RUST_BACKTRACE=full to include source snippets.
@@ -551,7 +555,7 @@ fn test_init_core_hooks_path_warning() -> eyre::Result<()> {
         insta::assert_snapshot!(stdout, @r###"
         Created config file at <repo-path>/.git/branchless/config
         Auto-detected your main branch as: master
-        If this is incorrect, run: git config branchless.core.mainBranch <branch>
+        If this is incorrect, run: git branchless init --main-branch <branch>
         Installing hook: post-commit
         Installing hook: post-merge
         Installing hook: post-rewrite
@@ -567,3 +571,29 @@ fn test_init_core_hooks_path_warning() -> eyre::Result<()> {
 
     Ok(())
 }
+
+#[test]
+fn test_init_worktree() -> eyre::Result<()> {
+    let git = make_git()?;
+    git.init_repo_with_options(&GitInitOptions {
+        run_branchless_init: false,
+        make_initial_commit: true,
+    })?;
+    git.commit_file("test1", 1)?;
+    git.commit_file("test2", 2)?;
+
+    let GitWorktreeWrapper {
+        temp_dir: _temp_dir,
+        worktree,
+    } = make_git_worktree(&git, "new-worktree")?;
+    worktree.run(&["branchless", "init"])?;
+    {
+        let (stdout, _stderr) = worktree.run(&["smartlog"])?;
+        insta::assert_snapshot!(stdout, @r###"
+        :
+        @ 96d1c37 (> new-worktree, master) create test2.txt
+        "###);
+    }
+
+    Ok(())
+}
