@@ -280,7 +280,7 @@ fn test_next_no_more_children() -> eyre::Result<()> {
 
 #[test]
 #[cfg(unix)]
-fn test_checkout_pty() -> eyre::Result<()> {
+fn test_switch_pty() -> eyre::Result<()> {
     let git = make_git()?;
 
     git.init_repo()?;
@@ -293,7 +293,7 @@ fn test_checkout_pty() -> eyre::Result<()> {
 
     run_in_pty(
         &git,
-        &["branchless", "checkout"],
+        &["branchless", "switch"],
         &[
             PtyAction::WaitUntilContains("> "),
             PtyAction::Write("test1"),
@@ -317,7 +317,7 @@ fn test_checkout_pty() -> eyre::Result<()> {
 
     run_in_pty(
         &git,
-        &["branchless", "checkout"],
+        &["branchless", "switch"],
         &[
             PtyAction::WaitUntilContains("> "),
             PtyAction::Write("test3"),
@@ -344,7 +344,7 @@ fn test_checkout_pty() -> eyre::Result<()> {
 
 #[test]
 #[cfg(unix)]
-fn test_checkout_abort() -> eyre::Result<()> {
+fn test_switch_abort() -> eyre::Result<()> {
     let git = make_git()?;
 
     git.init_repo()?;
@@ -354,7 +354,7 @@ fn test_checkout_abort() -> eyre::Result<()> {
 
     run_in_pty(
         &git,
-        &["branchless", "checkout"],
+        &["branchless", "switch"],
         &[
             PtyAction::WaitUntilContains("> "),
             PtyAction::Write(END_OF_TEXT),
@@ -555,7 +555,7 @@ fn test_navigation_failed_to_check_out_commit() -> eyre::Result<()> {
     git.commit_file("test2", 2)?;
     git.run(&["checkout", "HEAD^"])?;
 
-    git.write_file("test2", "conflicting contents")?;
+    git.write_file_txt("test2", "conflicting contents")?;
 
     {
         let (stdout, _stderr) = git.run_with_options(
@@ -576,7 +576,7 @@ fn test_navigation_failed_to_check_out_commit() -> eyre::Result<()> {
 
 #[test]
 #[cfg(unix)]
-fn test_checkout_pty_branch() -> eyre::Result<()> {
+fn test_switch_pty_branch() -> eyre::Result<()> {
     let git = make_git()?;
 
     git.init_repo()?;
@@ -588,7 +588,7 @@ fn test_checkout_pty_branch() -> eyre::Result<()> {
 
     run_in_pty(
         &git,
-        &["branchless", "checkout"],
+        &["branchless", "switch"],
         &[
             PtyAction::WaitUntilContains("> "),
             PtyAction::Write("master"),
@@ -600,7 +600,7 @@ fn test_checkout_pty_branch() -> eyre::Result<()> {
         let (stdout, _stderr) = git.run(&["smartlog"])?;
         insta::assert_snapshot!(stdout, @r###"
         :
-        @ 62fc20d (master) create test1.txt
+        @ 62fc20d (> master) create test1.txt
         |\
         | o 96d1c37 create test2.txt
         |
@@ -613,7 +613,7 @@ fn test_checkout_pty_branch() -> eyre::Result<()> {
 
 #[test]
 #[cfg(unix)]
-fn test_checkout_pty_initial_query() -> eyre::Result<()> {
+fn test_switch_pty_initial_query() -> eyre::Result<()> {
     let git = make_git()?;
 
     git.init_repo()?;
@@ -623,7 +623,7 @@ fn test_checkout_pty_initial_query() -> eyre::Result<()> {
 
     run_in_pty(
         &git,
-        &["branchless", "checkout", "-i", "test1"],
+        &["branchless", "switch", "-i", "test1"],
         &[
             PtyAction::WaitUntilContains("> test1"),
             PtyAction::Write(CARRIAGE_RETURN),
@@ -651,7 +651,7 @@ fn test_navigation_merge() -> eyre::Result<()> {
     git.detach_head()?;
     git.commit_file_with_contents("conflicting", 1, "foo\nbar\n")?;
     git.commit_file_with_contents("conflicting", 2, "baz\nqux\n")?;
-    git.write_file("conflicting", "foo\nbar\nqux\n")?;
+    git.write_file_txt("conflicting", "foo\nbar\nqux\n")?;
 
     {
         let (stdout, stderr) = git.run_with_options(
@@ -715,7 +715,7 @@ fn test_navigation_force() -> eyre::Result<()> {
     git.detach_head()?;
     git.commit_file_with_contents("conflicting", 1, "foo\nbar\n")?;
     git.commit_file_with_contents("conflicting", 2, "baz\nqux\n")?;
-    git.write_file("conflicting", "foo\n\nbar\nqux\n")?;
+    git.write_file_txt("conflicting", "foo\n\nbar\nqux\n")?;
 
     {
         let (stdout, stderr) = git.run_with_options(
@@ -759,7 +759,7 @@ fn test_navigation_force() -> eyre::Result<()> {
 }
 
 #[test]
-fn test_navigation_checkout_flags() -> eyre::Result<()> {
+fn test_navigation_switch_flags() -> eyre::Result<()> {
     let git = make_git()?;
     git.init_repo()?;
 
@@ -769,7 +769,7 @@ fn test_navigation_checkout_flags() -> eyre::Result<()> {
     {
         let git = git.duplicate_repo()?;
         {
-            let (stdout, _stderr) = git.run(&["branchless", "checkout", "-b", "foo", "HEAD^"])?;
+            let (stdout, _stderr) = git.run(&["branchless", "switch", "-c", "foo", "HEAD^"])?;
             insta::assert_snapshot!(stdout, @r###"
             branchless: running command: <git-executable> checkout HEAD^ -b foo
             :
@@ -780,7 +780,7 @@ fn test_navigation_checkout_flags() -> eyre::Result<()> {
         }
 
         {
-            let (stdout, _stderr) = git.run(&["branchless", "checkout", "-b", "bar"])?;
+            let (stdout, _stderr) = git.run(&["branchless", "switch", "-c", "bar"])?;
             insta::assert_snapshot!(stdout, @r###"
             branchless: running command: <git-executable> checkout -b bar
             :
@@ -795,8 +795,8 @@ fn test_navigation_checkout_flags() -> eyre::Result<()> {
         let git = git.duplicate_repo()?;
         {
             git.commit_file_with_contents("test1", 3, "new contents")?;
-            git.write_file("test1", "conflicting\n")?;
-            let (stdout, _stderr) = git.run(&["branchless", "checkout", "-f", "HEAD~2"])?;
+            git.write_file_txt("test1", "conflicting\n")?;
+            let (stdout, _stderr) = git.run(&["branchless", "switch", "-f", "HEAD~2"])?;
             insta::assert_snapshot!(stdout, @r###"
             branchless: running command: <git-executable> checkout HEAD~2 -f
             :
@@ -809,8 +809,8 @@ fn test_navigation_checkout_flags() -> eyre::Result<()> {
 
     {
         git.commit_file_with_contents("test1", 3, "new contents")?;
-        git.write_file("test1", "conflicting\n")?;
-        let (stdout, _stderr) = git.run(&["branchless", "checkout", "-m", "HEAD~2"])?;
+        git.write_file_txt("test1", "conflicting\n")?;
+        let (stdout, _stderr) = git.run(&["branchless", "switch", "-m", "HEAD~2"])?;
         insta::assert_snapshot!(stdout, @r###"
         branchless: running command: <git-executable> checkout HEAD~2 -m
         M	test1.txt
@@ -825,12 +825,12 @@ fn test_navigation_checkout_flags() -> eyre::Result<()> {
 }
 
 #[test]
-fn test_navigation_checkout_target_only() -> eyre::Result<()> {
+fn test_navigation_switch_target_only() -> eyre::Result<()> {
     let git = make_git()?;
     git.init_repo()?;
 
     {
-        let (stdout, _stderr) = git.run(&["branchless", "checkout", "master"])?;
+        let (stdout, _stderr) = git.run(&["branchless", "switch", "master"])?;
         insta::assert_snapshot!(stdout, @r###"
         branchless: running command: <git-executable> checkout master
         @ f777ecc (> master) create initial.txt
@@ -839,3 +839,83 @@ fn test_navigation_checkout_target_only() -> eyre::Result<()> {
 
     Ok(())
 }
+
+#[test]
+#[cfg(unix)]
+fn test_switch_auto_switch_interactive() -> eyre::Result<()> {
+    let git = make_git()?;
+
+    git.init_repo()?;
+    git.detach_head()?;
+    git.commit_file("test1", 1)?;
+    git.run(&["branch", "test1"])?;
+    git.commit_file("test2", 2)?;
+    git.run(&["branch", "test2"])?;
+
+    run_in_pty(
+        &git,
+        &["sw", "--interactive"],
+        &[
+            PtyAction::WaitUntilContains("> "),
+            PtyAction::Write("test1"),
+            PtyAction::WaitUntilContains("> 62fc20d"),
+            PtyAction::Write(CARRIAGE_RETURN),
+        ],
+    )?;
+
+    {
+        let (stdout, _stderr) = git.run(&["smartlog"])?;
+        insta::assert_snapshot!(stdout, @r###"
+        O f777ecc (master) create initial.txt
+        |
+        @ 62fc20d (> test1) create test1.txt
+        |
+        o 96d1c37 (test2) create test2.txt
+        "###);
+    }
+
+    Ok(())
+}
+
+#[test]
+#[cfg(unix)]
+fn test_switch_auto_switch_interactive_disabled() -> eyre::Result<()> {
+    let git = make_git()?;
+
+    git.init_repo()?;
+    git.run(&[
+        "config",
+        "branchless.navigation.autoSwitchBranches",
+        "false",
+    ])?;
+
+    git.detach_head()?;
+    git.commit_file("test1", 1)?;
+    git.run(&["branch", "test1"])?;
+    git.commit_file("test2", 2)?;
+    git.run(&["branch", "test2"])?;
+
+    run_in_pty(
+        &git,
+        &["sw", "--interactive"],
+        &[
+            PtyAction::WaitUntilContains("> "),
+            PtyAction::Write("test1"),
+            PtyAction::WaitUntilContains("> 62fc20d"),
+            PtyAction::Write(CARRIAGE_RETURN),
+        ],
+    )?;
+
+    {
+        let (stdout, _stderr) = git.run(&["smartlog"])?;
+        insta::assert_snapshot!(stdout, @r###"
+        O f777ecc (master) create initial.txt
+        |
+        @ 62fc20d (test1) create test1.txt
+        |
+        o 96d1c37 (test2) create test2.txt
+        "###);
+    }
+
+    Ok(())
+}
