@@ -83,9 +83,8 @@ fn test_query_eval_error() -> eyre::Result<()> {
                 ..Default::default()
             },
         )?;
-        insta::assert_snapshot!(stderr, @r###"
-        Evaluation error for expression 'foo()': no function with the name 'foo' could be found; these functions are available: all, ancestors, ancestors.nth, author.date, author.email, author.name, branches, children, committer.date, committer.email, committer.name, descendants, difference, draft, exactly, heads, intersection, message, none, not, only, parents, parents.nth, paths.changed, range, roots, stack, union
-        "###);
+        insta::assert_snapshot!(stderr, @"Evaluation error for expression 'foo()': no function with the name 'foo' could be found; these functions are available: all, ancestors, ancestors.nth, author.date, author.email, author.name, branches, children, committer.date, committer.email, committer.name, current, descendants, difference, draft, exactly, heads, intersection, main, message, none, not, only, parents, parents.nth, paths.changed, public, range, roots, stack, union
+");
         insta::assert_snapshot!(stdout, @"");
     }
 
@@ -183,6 +182,16 @@ fn test_query_hidden_commits() -> eyre::Result<()> {
     git.run(&["hide", "HEAD"])?;
     git.run(&["checkout", &test2_oid.to_string()])?;
 
+    {
+        let (stdout, stderr) = git.run(&["query", &format!("{}::", test2_oid)])?;
+        insta::assert_snapshot!(stderr, @"");
+        insta::assert_snapshot!(stdout, @r###"
+        96d1c37 create test2.txt
+        70deb1e create test3.txt
+        "###);
+    }
+
+    git.run(&["checkout", "-B", "master"])?;
     {
         let (stdout, stderr) = git.run(&["query", &format!("{}::", test2_oid)])?;
         insta::assert_snapshot!(stderr, @"");
