@@ -161,16 +161,13 @@ fn test_amended_initial_commit() -> eyre::Result<()> {
     git.run(&["commit", "--amend", "-m", "new initial commit"])?;
 
     {
+        // FIXME: there is no topological relationship between the new initial commit and `master`,
+        // so they shouldn't be connected in the smartlog.
         let (stdout, _stderr) = git.run(&["smartlog"])?;
         insta::assert_snapshot!(stdout, @r###"
         @ 9a9f929 new initial commit
-
-        X f777ecc (rewritten as 9a9f929a) create initial.txt
-        |
+        :
         O 62fc20d (master) create test1.txt
-        hint: there is 1 abandoned commit in your commit graph
-        hint: to fix this, run: git restack
-        hint: disable this hint by running: git config --global branchless.hint.smartlogFixAbandoned false
         "###);
     }
 
@@ -237,7 +234,7 @@ fn test_restack_aborts_during_rebase_conflict() -> eyre::Result<()> {
     git.commit_file("test2", 2)?;
     git.run(&["prev"])?;
 
-    git.write_file("test2", "conflicting test2 contents")?;
+    git.write_file_txt("test2", "conflicting test2 contents")?;
     git.run(&["add", "."])?;
     git.run(&["commit", "--amend", "-m", "amend test1 with test2 conflict"])?;
 
