@@ -134,17 +134,50 @@ fn test_merge_commit() -> eyre::Result<()> {
         O f777ecc (master) create initial.txt
         |\
         | o 62fc20d (test1) create test1.txt
-        | |
-        | @ fa4e4e1 (> test2and3) Merge branch 'test1' into test2and3
+        | & (merge) fa4e4e1 (> test2and3) Merge branch 'test1' into test2and3
         |
         o fe65c1f create test2.txt
         |
         o 0206717 create test3.txt
         |
+        | & (merge) 62fc20d (test1) create test1.txt
+        |/
         @ fa4e4e1 (> test2and3) Merge branch 'test1' into test2and3
         "###);
     }
 
+    git.run(&["checkout", "-b", "test4", "master"])?;
+    git.commit_file("test4", 4)?;
+    git.commit_file("test5", 5)?;
+    git.run(&["merge", "test1", "test2and3"])?;
+
+    {
+        let (stdout, _stderr) = git.run(&["smartlog"])?;
+        insta::assert_snapshot!(stdout, @r###"
+        O f777ecc (master) create initial.txt
+        |\
+        | o 62fc20d (test1) create test1.txt
+        | & (merge) fa4e4e1 (test2and3) Merge branch 'test1' into test2and3
+        |\
+        | o fe65c1f create test2.txt
+        | |
+        | o 0206717 create test3.txt
+        | |
+        | | & (merge) 62fc20d (test1) create test1.txt
+        | |/
+        | o fa4e4e1 (test2and3) Merge branch 'test1' into test2and3
+        | & (merge) 36a25e8 (> test4) Merge branch 'test2and3' into test4
+        |
+        o 8f7aef5 create test4.txt
+        |
+        o 47d30fa create test5.txt
+        |
+        | & (merge) fa4e4e1 (test2and3) Merge branch 'test1' into test2and3
+        |/
+        @ 36a25e8 (> test4) Merge branch 'test2and3' into test4
+        "###);
+    }
+
     Ok(())
 }
 
@@ -303,62 +336,12 @@ fn test_custom_main_branch() -> eyre::Result<()> {
     Ok(())
 }
 
-#[test]
-fn test_main_remote_branch() -> eyre::Result<()> {
-    let GitWrapperWithRemoteRepo {
-        temp_dir: _guard,
-        original_repo,
-        cloned_repo,
-    } = make_git_with_remote_repo()?;
-
-    {
-        original_repo.init_repo()?;
-        original_repo.commit_file("test1", 1)?;
-        original_repo.run(&[
-            "clone",
-            original_repo.repo_path.to_str().unwrap(),
-            cloned_repo.repo_path.to_str().unwrap(),
-        ])?;
-    }
-
-    {
-        cloned_repo.init_repo_with_options(&GitInitOptions {
-            make_initial_commit: false,
-            ..Default::default()
-        })?;
-        cloned_repo.detach_head()?;
-        cloned_repo.run(&["config", "branchless.core.mainBranch", "origin/master"])?;
-        cloned_repo.run(&["branch", "-d", "master"])?;
-        let (stdout, _stderr) = cloned_repo.run(&["smartlog"])?;
-        insta::assert_snapshot!(stdout, @r###"
-        :
-        @ 62fc20d (remote origin/master) create test1.txt
-        "###);
-    }
-
-    {
-        original_repo.commit_file("test2", 2)?;
-    }
-
-    {
-        cloned_repo.run(&["fetch"])?;
-        let (stdout, _stderr) = cloned_repo.run(&["smartlog"])?;
-        insta::assert_snapshot!(stdout, @r###"
-        :
-        @ 62fc20d create test1.txt
-        |
-        O 96d1c37 (remote origin/master) create test2.txt
-        "###);
-    }
-
-    Ok(())
-}
-
 #[test]
 fn test_show_rewritten_commit_hash() -> eyre::Result<()> {
     let git = make_git()?;
 
     git.init_repo()?;
+    git.detach_head()?;
     git.commit_file("test1", 1)?;
     git.commit_file("test2", 2)?;
     git.run(&["prev"])?;
@@ -368,13 +351,13 @@ fn test_show_rewritten_commit_hash() -> eyre::Result<()> {
     {
         let (stdout, _stderr) = git.run(&["smartlog"])?;
         insta::assert_snapshot!(stdout, @r###"
-        O f777ecc create initial.txt
+        O f777ecc (master) create initial.txt
         |\
         | @ 2ebe095 test1 version 2
         |
-        X 62fc20d (rewritten as 2ebe0950) create test1.txt
+        x 62fc20d (rewritten as 2ebe0950) create test1.txt
         |
-        O 96d1c37 (master) create test2.txt
+        o 96d1c37 create test2.txt
         hint: there is 1 abandoned commit in your commit graph
         hint: to fix this, run: git restack
         hint: disable this hint by running: git config --global branchless.hint.smartlogFixAbandoned false
@@ -539,19 +522,7 @@ fn test_active_non_head_main_branch_commit() -> eyre::Result<()> {
         let (stdout, _stderr) = cloned_repo.run(&["smartlog"])?;
         insta::assert_snapshot!(stdout, @r###"
         :
-        @ 70deb1e (> master, remote origin/master) create test3.txt
-        "###);
-    }
-
-    {
-        // Verify that both `origin/master` and `master` appear in the smartlog.
-        cloned_repo.commit_file("test4", 4)?;
-        let (stdout, _stderr) = cloned_repo.run(&["smartlog"])?;
-        insta::assert_snapshot!(stdout, @r###"
-        :
-        O 70deb1e (remote origin/master) create test3.txt
-        |
-        @ 355e173 (> master) create test4.txt
+        @ 70deb1e (> master) create test3.txt
         "###);
     }
 
@@ -567,6 +538,7 @@ fn test_smartlog_hint_abandoned() -> eyre::Result<()> {
     }
     git.init_repo()?;
 
+    git.detach_head()?;
     git.commit_file("test1", 1)?;
     git.commit_file("test2", 2)?;
     git.run(&["checkout", "HEAD^"])?;
@@ -575,13 +547,13 @@ fn test_smartlog_hint_abandoned() -> eyre::Result<()> {
     let hint_command = {
         let (stdout, _stderr) = git.run(&["smartlog"])?;
         insta::assert_snapshot!(stdout, @r###"
-        O f777ecc create initial.txt
+        O f777ecc (master) create initial.txt
         |\
         | @ ae94dc2 amended test1
         |
-        X 62fc20d (rewritten as ae94dc2a) create test1.txt
+        x 62fc20d (rewritten as ae94dc2a) create test1.txt
         |
-        O 96d1c37 (master) create test2.txt
+        o 96d1c37 create test2.txt
         hint: there is 1 abandoned commit in your commit graph
         hint: to fix this, run: git restack
         hint: disable this hint by running: git config --global branchless.hint.smartlogFixAbandoned false
@@ -594,13 +566,13 @@ fn test_smartlog_hint_abandoned() -> eyre::Result<()> {
     {
         let (stdout, _stderr) = git.run(&["smartlog"])?;
         insta::assert_snapshot!(stdout, @r###"
-        O f777ecc create initial.txt
+        O f777ecc (master) create initial.txt
         |\
         | @ ae94dc2 amended test1
         |
-        X 62fc20d (rewritten as ae94dc2a) create test1.txt
+        x 62fc20d (rewritten as ae94dc2a) create test1.txt
         |
-        O 96d1c37 (master) create test2.txt
+        o 96d1c37 create test2.txt
         "###);
     }
 
@@ -633,3 +605,175 @@ fn test_smartlog_hint_abandoned_except_current_commit() -> eyre::Result<()> {
 
     Ok(())
 }
+
+#[test]
+fn test_smartlog_sparse() -> eyre::Result<()> {
+    let git = make_git()?;
+
+    git.init_repo()?;
+    git.detach_head()?;
+    git.commit_file("test1", 1)?;
+    git.run(&["checkout", "master"])?;
+    git.commit_file("test2", 2)?;
+    git.commit_file("test3", 3)?;
+    git.detach_head()?;
+    git.commit_file("test4", 4)?;
+
+    {
+        let (stdout, _stderr) = git.run(&["smartlog", "none()"])?;
+        insta::assert_snapshot!(stdout, @r###"
+        :
+        O 0206717 (master) create test3.txt
+        |
+        @ 8e62740 create test4.txt
+        "###);
+    }
+
+    Ok(())
+}
+
+#[test]
+fn test_smartlog_sparse_branch() -> eyre::Result<()> {
+    let git = make_git()?;
+
+    git.init_repo()?;
+    git.detach_head()?;
+    git.commit_file("test1", 1)?;
+    let test2_oid = git.commit_file("test2", 2)?;
+    git.run(&["checkout", "master"])?;
+    git.commit_file("test3", 3)?;
+    git.commit_file("test4", 4)?;
+    git.detach_head()?;
+    git.commit_file("test5", 5)?;
+
+    {
+        let (stdout, _stderr) = git.run(&["smartlog", &test2_oid.to_string()])?;
+        insta::assert_snapshot!(stdout, @r###"
+        O f777ecc create initial.txt
+        |\
+        : # 1 omitted commit
+        : :
+        : o 96d1c37 create test2.txt
+        :
+        O 2b633ed (master) create test4.txt
+        |
+        @ 1393298 create test5.txt
+        "###);
+    }
+
+    Ok(())
+}
+
+#[test]
+fn test_smartlog_sparse_false_head() -> eyre::Result<()> {
+    let git = make_git()?;
+
+    git.init_repo()?;
+    git.detach_head()?;
+    git.commit_file("test1", 1)?;
+    let test2_oid = git.commit_file("test2", 2)?;
+    git.commit_file("test3", 3)?;
+    git.run(&["checkout", "master"])?;
+    git.commit_file("test4", 4)?;
+    git.detach_head()?;
+    git.commit_file("test5", 5)?;
+    git.commit_file("test6", 6)?;
+
+    {
+        let (stdout, _stderr) = git.run(&["smartlog", &test2_oid.to_string()])?;
+        insta::assert_snapshot!(stdout, @r###"
+        O f777ecc create initial.txt
+        |\
+        | # 1 omitted commit
+        | :
+        | o 96d1c37 create test2.txt
+        | :
+        | # 1 omitted descendant commit
+        |
+        O 8f7aef5 (master) create test4.txt
+        :
+        # 1 omitted commit
+        :
+        @ 68975e5 create test6.txt
+        "###);
+    }
+
+    Ok(())
+}
+
+#[test]
+fn test_smartlog_sparse_main_false_head() -> eyre::Result<()> {
+    let git = make_git()?;
+
+    git.init_repo()?;
+    git.commit_file("test1", 1)?;
+    git.detach_head()?;
+    git.commit_file("test2", 2)?;
+    git.run(&["checkout", "HEAD~"])?;
+
+    {
+        let (stdout, _stderr) = git.run(&["smartlog", "none()"])?;
+        insta::assert_snapshot!(stdout, @r###"
+        :
+        @ 62fc20d (master) create test1.txt
+        :
+        # 1 omitted descendant commit
+        "###);
+    }
+
+    Ok(())
+}
+
+#[test]
+fn test_smartlog_hidden() -> eyre::Result<()> {
+    let git = make_git()?;
+    git.init_repo()?;
+
+    git.detach_head()?;
+    git.commit_file("test1", 1)?;
+    git.run(&["commit", "--amend", "-m", "amended test1"])?;
+
+    {
+        let (stdout, _stderr) = git.run(&["smartlog", "--hidden"])?;
+        insta::assert_snapshot!(stdout, @r###"
+        O f777ecc (master) create initial.txt
+        |\
+        | @ ae94dc2 amended test1
+        |
+        x 62fc20d (rewritten as ae94dc2a) create test1.txt
+        "###);
+    }
+
+    Ok(())
+}
+
+#[test]
+fn test_smartlog_sparse_vertical_ellipsis_sibling_commits() -> eyre::Result<()> {
+    let git = make_git()?;
+    git.init_repo()?;
+
+    git.detach_head()?;
+    git.commit_file("test1", 1)?;
+    git.run(&["checkout", "HEAD~"])?;
+    git.commit_file("test2", 2)?;
+    git.run(&["checkout", "HEAD~"])?;
+    git.commit_file("test3", 3)?;
+    git.commit_file("test4", 4)?;
+
+    {
+        let (stdout, _stderr) = git.run(&["smartlog", "heads(draft())"])?;
+        insta::assert_snapshot!(stdout, @r###"
+        O f777ecc (master) create initial.txt
+        |\
+        | o 62fc20d create test1.txt
+        |\
+        | o fe65c1f create test2.txt
+        :
+        # 1 omitted commit
+        :
+        @ 2b633ed create test4.txt
+        "###);
+    }
+
+    Ok(())
+}
