@@ -13,7 +13,7 @@ use lib::core::eventlog::{EventCursor, EventLogDb, EventReplayer};
 use lib::core::formatting::Glyphs;
 use lib::core::repo_ext::RepoExt;
 use lib::git::{GitRunInfo, GitVersion, Repo};
-use lib::testing::{make_git, Git};
+use lib::testing::{make_git, Git, GitInitOptions};
 
 use cursive::event::Key;
 use cursive::CursiveRunnable;
@@ -412,7 +412,7 @@ fn test_undo_move_refs() -> eyre::Result<()> {
                                 to 62fc20d create test1.txt
         3. Check out from 96d1c37 create test2.txt
                        to 62fc20d create test1.txt
-        Confirm? [yN] branchless: running command: <git-executable> checkout 62fc20d2a290daea0d52bdc2ed2ad4be6491010e --detach
+        Confirm? [yN] branchless: running command: <git-executable> checkout master --detach
         Applied 3 inverse events.
         "###);
         assert_eq!(exit_code, 0);
@@ -637,7 +637,7 @@ fn test_undo_doesnt_make_working_dir_dirty() -> eyre::Result<()> {
                        to f777ecc create initial.txt
         5. Delete branch foo at f777ecc create initial.txt
 
-        Confirm? [yN] branchless: running command: <git-executable> checkout f777ecc9b0db5ed372b2615695191a8a17f79f24 --detach
+        Confirm? [yN] branchless: running command: <git-executable> checkout master --detach
         Applied 5 inverse events.
         "###);
         assert_eq!(exit_code, 0);
@@ -876,7 +876,7 @@ fn test_undo_noninteractive() -> eyre::Result<()> {
                                 to 96d1c37 create test2.txt
         4. Check out from 9ed8f9a bad message
                        to 96d1c37 create test2.txt
-        Confirm? [yN] branchless: running command: <git-executable> checkout 96d1c37a3d4363611c49f7e52186e189a04c531f --detach
+        Confirm? [yN] branchless: running command: <git-executable> checkout master --detach
         :
         @ 96d1c37 (master) create test2.txt
         Applied 4 inverse events.
@@ -918,3 +918,88 @@ fn test_undo_no_confirm() -> eyre::Result<()> {
 
     Ok(())
 }
+
+#[test]
+fn test_undo_unseen_commit() -> eyre::Result<()> {
+    // Disabled since we no longer support `origin/master` as a main branch, but this test might be
+    // useful in the future.
+    if cfg!(none) {
+        let git = make_git()?;
+        if !git.supports_reference_transactions()? {
+            return Ok(());
+        }
+
+        git.init_repo_with_options(&GitInitOptions {
+            run_branchless_init: false,
+            make_initial_commit: true,
+        })?;
+        git.commit_file("test1", 1)?;
+        git.detach_head()?;
+        let test2_oid = git.commit_file("test2", 2)?;
+        git.run(&["checkout", "HEAD^"])?;
+        let test3_oid = git.commit_file("test3", 3)?;
+        git.run(&["checkout", "master"])?;
+        git.run(&["branchless", "init"])?;
+
+        // Move the remote-tracking branch to a commit (test3) which doesn't have the previous location
+        // (test2) as an ancestor, to ensure that the DAG won't have observed it.
+        git.run(&[
+            "update-ref",
+            "refs/remotes/origin/master",
+            &test2_oid.to_string(),
+        ])?;
+        git.run(&[
+            "update-ref",
+            "refs/remotes/origin/master",
+            &test3_oid.to_string(),
+            &test2_oid.to_string(),
+        ])?;
+
+        // Set the main branch to `origin/master` to ensure that it appears in historical smartlogs.
+        git.run(&["remote", "add", "origin", "file:///some-remote"])?;
+        git.run(&["branch", "-u", "origin/master"])?;
+        git.run(&["branchless", "init", "--main-branch", "origin/master"])?;
+
+        {
+            let screenshot1 = Default::default();
+            run_select_past_event(
+                &git.get_repo()?,
+                vec![
+                    CursiveTestingEvent::Event('p'.into()),
+                    CursiveTestingEvent::Event('p'.into()),
+                    CursiveTestingEvent::Event('p'.into()),
+                    CursiveTestingEvent::TakeScreenshot(Rc::clone(&screenshot1)),
+                    CursiveTestingEvent::Event('q'.into()),
+                ],
+            )?;
+            insta::assert_snapshot!(screen_to_string(&screenshot1), @r###"
+        ┌───────────────────────────────────────────────────┤ Commit graph ├───────────────────────────────────────────────────┐
+        │:                                                                                                                     │
+        │O 4838e49 (remote origin/master) create test3.txt                                                                     │
+        │                                                                                                                      │
+        │                                                                                                                      │
+        │                                                                                                                      │
+        │                                                                                                                      │
+        │                                                                                                                      │
+        │                                                                                                                      │
+        │                                                                                                                      │
+        │                                                                                                                      │
+        │                                                                                                                      │
+        │                                                                                                                      │
+        │                                                                                                                      │
+        │                                                                                                                      │
+        │                                                                                                                      │
+        │                                                                                                                      │
+        │                                                                                                                      │
+        │                                                                                                                      │
+        │                                                                                                                      │
+        └──────────────────────────────────────────────────────────────────────────────────────────────────────────────────────┘
+        ┌──────────────────────────────────────────────────────┤ Events ├──────────────────────────────────────────────────────┐
+        │There are no previous available events.                                                                               │
+        └──────────────────────────────────────────────────────────────────────────────────────────────────────────────────────┘
+        "###);
+        }
+    }
+
+    Ok(())
+}
